# Copyright 2015-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class BuildIdProfileFormatTestCase(TestCase):

	def testBuildIdProfileFormat(self):

		profile = {
			"packages": ("=app-misc/A-1-2::test_repo",),
			"package.mask": ("<app-misc/A-1::test_repo",),
			"package.keywords": ("app-misc/A-1::test_repo x86",),
			"package.unmask": (">=app-misc/A-1::test_repo",),
			"package.use": ("app-misc/A-1::test_repo foo",),
			"package.use.mask": ("app-misc/A-1::test_repo -foo",),
			"package.use.stable.mask": ("app-misc/A-1::test_repo -foo",),
			"package.use.force": ("app-misc/A-1::test_repo foo",),
			"package.use.stable.force": ("app-misc/A-1::test_repo foo",),
			"package.provided": ("sys-libs/zlib-1.2.8-r1",),
		}

		repo_configs = {
			"test_repo": {
				"layout.conf": (
					"profile-formats = build-id profile-repo-deps profile-set",
				),
			}
		}

		user_config = {
			"make.conf":
				(
					"FEATURES=\"binpkg-multi-instance\"",
				),
		}

		ebuilds = {
			"app-misc/A-1" : {
				"EAPI": "5",
				"RDEPEND": "sys-libs/zlib dev-libs/B[foo]",
				"DEPEND": "sys-libs/zlib dev-libs/B[foo]",
			},
			"dev-libs/B-1" : {
				"EAPI": "5",
				"IUSE": "foo",
			},
		}

		binpkgs = (
			("app-misc/A-1", {
				"EAPI": "5",
				"BUILD_ID": "1",
				"BUILD_TIME": "1",
				"RDEPEND": "sys-libs/zlib dev-libs/B[foo]",
				"DEPEND": "sys-libs/zlib dev-libs/B[foo]",
			}),
			("app-misc/A-1", {
				"EAPI": "5",
				"BUILD_ID": "2",
				"BUILD_TIME": "2",
				"RDEPEND": "sys-libs/zlib dev-libs/B[foo]",
				"DEPEND": "sys-libs/zlib dev-libs/B[foo]",
			}),
			("app-misc/A-1", {
				"EAPI": "5",
				"BUILD_ID": "3",
				"BUILD_TIME": "3",
				"RDEPEND": "sys-libs/zlib dev-libs/B[foo]",
				"DEPEND": "sys-libs/zlib dev-libs/B[foo]",
			}),
			("dev-libs/B-1", {
				"EAPI": "5",
				"IUSE": "foo",
				"USE": "",
				"BUILD_ID": "1",
				"BUILD_TIME": "1",
			}),
			("dev-libs/B-1", {
				"EAPI": "5",
				"IUSE": "foo",
				"USE": "foo",
				"BUILD_ID": "2",
				"BUILD_TIME": "2",
			}),
			("dev-libs/B-1", {
				"EAPI": "5",
				"IUSE": "foo",
				"USE": "",
				"BUILD_ID": "3",
				"BUILD_TIME": "3",
			}),
		)

		installed = {
			"app-misc/A-1" : {
				"EAPI": "5",
				"BUILD_ID": "1",
				"BUILD_TIME": "1",
				"RDEPEND": "sys-libs/zlib",
				"DEPEND": "sys-libs/zlib",
			},
			"dev-libs/B-1" : {
				"EAPI": "5",
				"IUSE": "foo",
				"USE": "foo",
				"BUILD_ID": "2",
				"BUILD_TIME": "2",
			},
		}

		world = ()

		test_cases = (

			ResolverPlaygroundTestCase(
				["@world"],
				options = {"--emptytree": True, "--usepkgonly": True},
				success = True,
				mergelist = [
					"[binary]dev-libs/B-1-2",
					"[binary]app-misc/A-1-2"
				]
			),

		)

		playground = ResolverPlayground(debug=False,
			binpkgs=binpkgs, ebuilds=ebuilds, installed=installed,
			repo_configs=repo_configs, profile=profile,
			user_config=user_config, world=world)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True,
					test_case.fail_msg)
		finally:
			# Disable debug so that cleanup works.
			#playground.debug = False
			playground.cleanup()



#!/usr/bin/python -b
# Copyright 2010-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2
#
# This is a helper which ebuild processes can use
# to communicate with portage's main python process.

# This block ensures that ^C interrupts are handled quietly.
try:
	import os
	import signal

	def exithandler(signum, _frame):
		signal.signal(signum, signal.SIG_DFL)
		os.kill(os.getpid(), signum)

	signal.signal(signal.SIGINT, exithandler)
	signal.signal(signal.SIGTERM, exithandler)
	signal.signal(signal.SIGPIPE, signal.SIG_DFL)

except KeyboardInterrupt:
	raise SystemExit(130)

import errno
import logging
import pickle
import platform
import sys
import time

def debug_signal(signum, frame):
	import pdb
	pdb.set_trace()

if platform.python_implementation() == 'Jython':
	debug_signum = signal.SIGUSR2 # bug #424259
else:
	debug_signum = signal.SIGUSR1

signal.signal(debug_signum, debug_signal)

if os.path.isfile(os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), ".portage_not_installed")):
	pym_paths = [os.path.join(os.path.dirname(os.path.dirname(os.path.realpath(__file__))), "lib")]
	sys.path.insert(0, pym_paths[0])
else:
	import sysconfig
	pym_paths = [
		os.path.join(sysconfig.get_path("purelib"), x) for x in ("_emerge", "portage")
	]
# Avoid sandbox violations after Python upgrade.
if os.environ.get("SANDBOX_ON") == "1":
	sandbox_write = os.environ.get("SANDBOX_WRITE", "").split(":")
	for pym_path in pym_paths:
		if pym_path not in sandbox_write:
			sandbox_write.append(pym_path)
			os.environ["SANDBOX_WRITE"] = ":".join(filter(None, sandbox_write))
	del pym_path, sandbox_write
del pym_paths

import portage
portage._internal_caller = True
portage._disable_legacy_globals()

from portage.util._eventloop.global_event_loop import global_event_loop
from _emerge.AbstractPollTask import AbstractPollTask
from _emerge.PipeReader import PipeReader

RETURNCODE_WRITE_FAILED = 2

class FifoWriter(AbstractPollTask):

	__slots__ = ('buf', 'fifo', '_fd')

	def _start(self):
		try:
			self._fd = os.open(self.fifo, os.O_WRONLY|os.O_NONBLOCK)
		except OSError as e:
			if e.errno == errno.ENXIO:
				# This happens if the daemon has been killed.
				self.returncode = RETURNCODE_WRITE_FAILED
				self._unregister()
				self._async_wait()
				return
			else:
				raise
		self.scheduler.add_writer(
			self._fd,
			self._output_handler)
		self._registered = True

	def _output_handler(self):
		# The whole buf should be able to fit in the fifo with
		# a single write call, so there's no valid reason for
		# os.write to raise EAGAIN here.
		fd = self._fd
		buf = self.buf
		while buf:
			try:
				buf = buf[os.write(fd, buf):]
			except EnvironmentError:
				self.returncode = RETURNCODE_WRITE_FAILED
				self._async_wait()
				return

		self.returncode = os.EX_OK
		self._async_wait()

	def _cancel(self):
		self.returncode = self._cancelled_returncode
		self._unregister()

	def _unregister(self):
		self._registered = False
		if self._fd is not None:
			self.scheduler.remove_writer(self._fd)
			os.close(self._fd)
			self._fd = None

class EbuildIpc:

	# Timeout for each individual communication attempt (we retry
	# as long as the daemon process appears to be alive).
	_COMMUNICATE_RETRY_TIMEOUT = 15 # seconds

	def __init__(self):
		self.fifo_dir = os.environ['PORTAGE_BUILDDIR']
		self.ipc_in_fifo = os.path.join(self.fifo_dir, '.ipc_in')
		self.ipc_out_fifo = os.path.join(self.fifo_dir, '.ipc_out')
		self.ipc_lock_file = os.path.join(self.fifo_dir, '.ipc_lock')

	def _daemon_is_alive(self):
		try:
			builddir_lock = portage.locks.lockfile(self.fifo_dir,
				wantnewlockfile=True, flags=os.O_NONBLOCK)
		except portage.exception.TryAgain:
			return True
		else:
			portage.locks.unlockfile(builddir_lock)
			return False

	def communicate(self, args):

		# Make locks quiet since unintended locking messages displayed on
		# stdout could corrupt the intended output of this program.
		portage.locks._quiet = True
		lock_obj = portage.locks.lockfile(self.ipc_lock_file, unlinkfile=True)

		try:
			return self._communicate(args)
		finally:
			portage.locks.unlockfile(lock_obj)

	def _timeout_retry_msg(self, start_time, when):
		time_elapsed = time.time() - start_time
		portage.util.writemsg_level(
			portage.localization._(
			'ebuild-ipc timed out %s after %d seconds,' + \
			' retrying...\n') % (when, time_elapsed),
			level=logging.ERROR, noiselevel=-1)

	def _no_daemon_msg(self):
		portage.util.writemsg_level(
			portage.localization._(
			'ebuild-ipc: daemon process not detected\n'),
			level=logging.ERROR, noiselevel=-1)

	def _run_writer(self, fifo_writer, msg):
		"""
		Wait on pid and return an appropriate exit code. This
		may return unsuccessfully due to timeout if the daemon
		process does not appear to be alive.
		"""

		start_time = time.time()

		fifo_writer.start()
		eof = fifo_writer.poll() is not None

		while not eof:
			fifo_writer._wait_loop(timeout=self._COMMUNICATE_RETRY_TIMEOUT)

			eof = fifo_writer.poll() is not None
			if eof:
				break
			elif self._daemon_is_alive():
				self._timeout_retry_msg(start_time, msg)
			else:
				fifo_writer.cancel()
				self._no_daemon_msg()
				fifo_writer.wait()
				return 2

		return fifo_writer.wait()

	def _receive_reply(self, input_fd):

		start_time = time.time()

		pipe_reader = PipeReader(input_files={"input_fd":input_fd},
			scheduler=global_event_loop())
		pipe_reader.start()

		eof = pipe_reader.poll() is not None

		while not eof:
			pipe_reader._wait_loop(timeout=self._COMMUNICATE_RETRY_TIMEOUT)
			eof = pipe_reader.poll() is not None
			if not eof:
				if self._daemon_is_alive():
					self._timeout_retry_msg(start_time,
						portage.localization._('during read'))
				else:
					pipe_reader.cancel()
					self._no_daemon_msg()
					return 2

		buf = pipe_reader.getvalue()

		retval = 2

		if not buf:

			portage.util.writemsg_level(
				"ebuild-ipc: %s\n" % \
				(portage.localization._('read failed'),),
				level=logging.ERROR, noiselevel=-1)

		else:

			try:
				reply = pickle.loads(buf)
			except SystemExit:
				raise
			except Exception as e:
				# The pickle module can raise practically
				# any exception when given corrupt data.
				portage.util.writemsg_level(
					"ebuild-ipc: %s\n" % (e,),
					level=logging.ERROR, noiselevel=-1)

			else:

				(out, err, retval) = reply

				if out:
					portage.util.writemsg_stdout(out, noiselevel=-1)

				if err:
					portage.util.writemsg(err, noiselevel=-1)

		return retval

	def _communicate(self, args):

		if not self._daemon_is_alive():
			self._no_daemon_msg()
			return 2

		# Open the input fifo before the output fifo, in order to make it
		# possible for the daemon to send a reply without blocking. This
		# improves performance, and also makes it possible for the daemon
		# to do a non-blocking write without a race condition.
		input_fd = os.open(self.ipc_out_fifo,
			os.O_RDONLY|os.O_NONBLOCK)

		# Use forks so that the child process can handle blocking IO
		# un-interrupted, while the parent handles all timeout
		# considerations. This helps to avoid possible race conditions
		# from interference between timeouts and blocking IO operations.
		msg = portage.localization._('during write')
		retval = self._run_writer(FifoWriter(buf=pickle.dumps(args),
			fifo=self.ipc_in_fifo, scheduler=global_event_loop()), msg)

		if retval != os.EX_OK:
			portage.util.writemsg_level(
				"ebuild-ipc: %s: %s\n" % (msg,
				portage.localization._('subprocess failure: %s') % \
				retval), level=logging.ERROR, noiselevel=-1)
			return retval

		if not self._daemon_is_alive():
			self._no_daemon_msg()
			return 2

		return self._receive_reply(input_fd)

def ebuild_ipc_main(args):
	ebuild_ipc = EbuildIpc()
	return ebuild_ipc.communicate(args)

if __name__ == '__main__':
	try:
		sys.exit(ebuild_ipc_main(sys.argv[1:]))
	finally:
		global_event_loop().close()



# Copyright 1999-2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import logging

import portage
from portage import os
from portage.util import writemsg_level
from portage.sync.syncbase import NewBase


class SVNSync(NewBase):
	'''SVN sync module'''

	short_desc = "Perform sync operations on SVN repositories"

	@staticmethod
	def name():
		return "SVNSync"


	def __init__(self):
		NewBase.__init__(self, "svn", "dev-vcs/subversion")


	def exists(self, **kwargs):
		'''Tests whether the repo actually exists'''
		return os.path.exists(os.path.join(self.repo.location, '.svn'))


	def new(self, **kwargs):
		if kwargs:
			self._kwargs(kwargs)
		#initial checkout
		svn_root = self.repo.sync_uri
		exitcode = portage.process.spawn_bash(
			"cd %s; exec svn co %s ." %
			(portage._shell_quote(self.repo.location),
			portage._shell_quote(svn_root)),
			**self.spawn_kwargs)
		if exitcode != os.EX_OK:
			msg = "!!! svn checkout error; exiting."
			self.logger(self.xterm_titles, msg)
			writemsg_level(msg + "\n", noiselevel=-1, level=logging.ERROR)
		return (exitcode, False)


	def update(self):
		"""
		Internal function to update an existing SVN repository

		@return: tuple of return code (0=success), whether the cache
			needs to be updated
		@rtype: (int, bool)
		"""

		exitcode = self._svn_upgrade()
		if exitcode != os.EX_OK:
			return (exitcode, False)

		#svn update
		exitcode = portage.process.spawn_bash(
			"cd %s; exec svn update" % \
			(portage._shell_quote(self.repo.location),),
			**self.spawn_kwargs)
		if exitcode != os.EX_OK:
			msg = "!!! svn update error; exiting."
			self.logger(self.xterm_titles, msg)
			writemsg_level(msg + "\n", noiselevel=-1, level=logging.ERROR)
		return (exitcode, False)


	def _svn_upgrade(self):
		"""
		Internal function which performs an svn upgrade on the repo

		@return: tuple of return code (0=success), whether the cache
			needs to be updated
		@rtype: (int, bool)
		"""
		exitcode = portage.process.spawn_bash(
			"cd %s; exec svn upgrade" %
			(portage._shell_quote(self.repo.location),),
			**self.spawn_kwargs)
		if exitcode != os.EX_OK:
			msg = "!!! svn upgrade error; exiting."
			self.logger(self.xterm_titles, msg)
			writemsg_level(msg + "\n", noiselevel=-1, level=logging.ERROR)
		return exitcode



# Copyright 1999-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import errno

from _emerge.CompositeTask import CompositeTask
from _emerge.SpawnProcess import SpawnProcess
from portage import os, _shell_quote, _unicode_encode
from portage.const import BASH_BINARY

class BinpkgEnvExtractor(CompositeTask):
	"""
	Extract environment.bz2 for a binary or installed package.
	"""
	__slots__ = ('settings',)

	def saved_env_exists(self):
		return os.path.exists(self._get_saved_env_path())

	def dest_env_exists(self):
		return os.path.exists(self._get_dest_env_path())

	def _get_saved_env_path(self):
		return os.path.join(os.path.dirname(self.settings['EBUILD']),
			"environment.bz2")

	def _get_dest_env_path(self):
		return os.path.join(self.settings["T"], "environment")

	def _start(self):
		saved_env_path = self._get_saved_env_path()
		dest_env_path = self._get_dest_env_path()
		shell_cmd = "${PORTAGE_BUNZIP2_COMMAND:-${PORTAGE_BZIP2_COMMAND} -d} -c -- %s > %s" % \
			(_shell_quote(saved_env_path),
			_shell_quote(dest_env_path))

		logfile = None
		if self.settings.get("PORTAGE_BACKGROUND") != "subprocess":
			logfile = self.settings.get("PORTAGE_LOG_FILE")

		extractor_proc = SpawnProcess(
			args=[BASH_BINARY, "-c", shell_cmd],
			background=self.background,
			env=self.settings.environ(),
			scheduler=self.scheduler,
			logfile=logfile)

		self._start_task(extractor_proc, self._extractor_exit)

	def _remove_dest_env(self):
		try:
			os.unlink(self._get_dest_env_path())
		except OSError as e:
			if e.errno != errno.ENOENT:
				raise

	def _extractor_exit(self, extractor_proc):

		if self._default_exit(extractor_proc) != os.EX_OK:
			self._remove_dest_env()
			self.wait()
			return

		# This is a signal to ebuild.sh, so that it knows to filter
		# out things like SANDBOX_{DENY,PREDICT,READ,WRITE} that
		# would be preserved between normal phases.
		open(_unicode_encode(self._get_dest_env_path() + '.raw'), 'wb').close()

		self._current_task = None
		self.returncode = os.EX_OK
		self.wait()



# Copyright 2010-2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

__all__ = ["dep_expand"]

import re

from portage.dbapi.cpv_expand import cpv_expand
from portage.dep import Atom, isvalidatom
from portage.exception import InvalidAtom
from portage.versions import catsplit

def dep_expand(mydep, mydb=None, use_cache=1, settings=None):
	'''
	@rtype: Atom
	'''
	orig_dep = mydep
	if isinstance(orig_dep, Atom):
		has_cat = True
	else:
		if not mydep:
			return mydep
		if mydep[0] == "*":
			mydep = mydep[1:]
			orig_dep = mydep
		has_cat = '/' in orig_dep.split(':')[0]
		if not has_cat:
			alphanum = re.search(r'\w', orig_dep)
			if alphanum:
				mydep = orig_dep[:alphanum.start()] + "null/" + \
					orig_dep[alphanum.start():]
		try:
			mydep = Atom(mydep, allow_repo=True)
		except InvalidAtom:
			# Missing '=' prefix is allowed for backward compatibility.
			if not isvalidatom("=" + mydep, allow_repo=True):
				raise
			mydep = Atom('=' + mydep, allow_repo=True)
			orig_dep = '=' + orig_dep
		if not has_cat:
			null_cat, pn = catsplit(mydep.cp)
			mydep = pn

	if has_cat:
		# Optimize most common cases to avoid calling cpv_expand.
		if not mydep.cp.startswith("virtual/"):
			return mydep
		if not hasattr(mydb, "cp_list") or \
			mydb.cp_list(mydep.cp):
			return mydep
		# Fallback to legacy cpv_expand for old-style PROVIDE virtuals.
		mydep = mydep.cp

	expanded = cpv_expand(mydep, mydb=mydb,
		use_cache=use_cache, settings=settings)
	return Atom(orig_dep.replace(mydep, expanded, 1), allow_repo=True)



# Copyright 2012-2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

try:
	import threading
except ImportError:
	# dummy_threading will not suffice
	threading = None

from portage import os
from _emerge.AbstractPollTask import AbstractPollTask

class PipeReaderBlockingIO(AbstractPollTask):
	"""
	Reads output from one or more files and saves it in memory, for
	retrieval via the getvalue() method. This is driven by a thread
	for each input file, in order to support blocking IO.  This may
	be useful for using threads to handle blocking IO with Jython,
	since Jython lacks the fcntl module which is needed for
	non-blocking IO (see http://bugs.jython.org/issue1074).
	"""

	__slots__ = ("input_files", "_read_data", "_terminate",
		"_threads", "_thread_rlock")

	def _start(self):
		self._terminate = threading.Event()
		self._threads = {}
		self._read_data = []

		self._registered = True
		self._thread_rlock = threading.RLock()
		with self._thread_rlock:
			for f in self.input_files.values():
				t = threading.Thread(target=self._reader_thread, args=(f,))
				t.daemon = True
				t.start()
				self._threads[f] = t

	def _reader_thread(self, f):
		try:
			terminated = self._terminate.is_set
		except AttributeError:
			# Jython 2.7.0a2
			terminated = self._terminate.isSet
		bufsize = self._bufsize
		while not terminated():
			buf = f.read(bufsize)
			with self._thread_rlock:
				if terminated():
					break
				elif buf:
					self._read_data.append(buf)
				else:
					del self._threads[f]
					if not self._threads:
						# Thread-safe callback to EventLoop
						self.scheduler.call_soon_threadsafe(self._eof)
					break
		f.close()

	def _eof(self):
		self._registered = False
		if self.returncode is None:
			self.returncode = os.EX_OK
		self._async_wait()

	def _cancel(self):
		self._terminate.set()
		self._registered = False
		if self.returncode is None:
			self.returncode = self._cancelled_returncode
		self._async_wait()

	def getvalue(self):
		"""Retrieve the entire contents"""
		with self._thread_rlock:
			return b''.join(self._read_data)

	def close(self):
		"""Free the memory buffer."""
		with self._thread_rlock:
			self._read_data = None



# Copyright 2006-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.dep import Atom
from portage.exception import InvalidAtom

class TestAtom(TestCase):

	def testAtom(self):

		tests = (
			("=sys-apps/portage-2.1-r1:0[doc,a=,!b=,c?,!d?,-e]",
				('=',  'sys-apps/portage', '2.1-r1', '0', '[doc,a=,!b=,c?,!d?,-e]', None), False, False),
			("=sys-apps/portage-2.1-r1*:0[doc]",
				('=*',  'sys-apps/portage', '2.1-r1', '0', '[doc]', None), False, False),
			("sys-apps/portage:0[doc]",
				(None,  'sys-apps/portage', None, '0', '[doc]', None), False, False),
			("sys-apps/portage:0[doc]",
				(None,  'sys-apps/portage', None, '0', '[doc]', None), False, False),
			("*/*",
				(None,  '*/*', None, None, None, None), True, False),
			("=*/*-*9999*",
				('=*',  '*/*', '*9999*', None, None, None), True, False),
			("=*/*-*9999*:0::repo_name",
				('=*',  '*/*', '*9999*', '0', None, 'repo_name'), True, True),
			("=*/*-*_beta*",
				('=*',  '*/*', '*_beta*', None, None, None), True, False),
			("=*/*-*_beta*:0::repo_name",
				('=*',  '*/*', '*_beta*', '0', None, 'repo_name'), True, True),
			("sys-apps/*",
				(None,  'sys-apps/*', None, None, None, None), True, False),
			("*/portage",
				(None,  '*/portage', None, None, None, None), True, False),
			("s*s-*/portage:1",
				(None,  's*s-*/portage', None, '1', None, None), True, False),
			("*/po*ge:2",
				(None,  '*/po*ge', None, '2', None, None), True, False),
			("!dev-libs/A",
				(None,  'dev-libs/A', None, None, None, None), True, True),
			("!!dev-libs/A",
				(None,  'dev-libs/A', None, None, None, None), True, True),
			("!!dev-libs/A",
				(None,  'dev-libs/A', None, None, None, None), True, True),
			("dev-libs/A[foo(+)]",
				(None,  'dev-libs/A', None, None, "[foo(+)]", None), True, True),
			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]",
				(None,  'dev-libs/A', None, None, "[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", None), True, True),
			("dev-libs/A:2[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]",
				(None,  'dev-libs/A', None, "2", "[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", None), True, True),

			("=sys-apps/portage-2.1-r1:0::repo_name[doc,a=,!b=,c?,!d?,-e]",
				('=',  'sys-apps/portage', '2.1-r1', '0', '[doc,a=,!b=,c?,!d?,-e]', 'repo_name'), False, True),
			("=sys-apps/portage-2.1-r1*:0::repo_name[doc]",
				('=*',  'sys-apps/portage', '2.1-r1', '0', '[doc]', 'repo_name'), False, True),
			("sys-apps/portage:0::repo_name[doc]",
				(None,  'sys-apps/portage', None, '0', '[doc]', 'repo_name'), False, True),

			("*/*::repo_name",
				(None,  '*/*', None, None, None, 'repo_name'), True, True),
			("sys-apps/*::repo_name",
				(None,  'sys-apps/*', None, None, None, 'repo_name'), True, True),
			("*/portage::repo_name",
				(None,  '*/portage', None, None, None, 'repo_name'), True, True),
			("s*s-*/portage:1::repo_name",
				(None,  's*s-*/portage', None, '1', None, 'repo_name'), True, True),
		)

		tests_xfail = (
			(Atom("sys-apps/portage"), False, False),
			("cat/pkg[a!]", False, False),
			("cat/pkg[!a]", False, False),
			("cat/pkg[!a!]", False, False),
			("cat/pkg[!a-]", False, False),
			("cat/pkg[-a=]", False, False),
			("cat/pkg[-a?]", False, False),
			("cat/pkg[-a!]", False, False),
			("cat/pkg[=a]", False, False),
			("cat/pkg[=a=]", False, False),
			("cat/pkg[=a?]", False, False),
			("cat/pkg[=a!]", False, False),
			("cat/pkg[=a-]", False, False),
			("cat/pkg[?a]", False, False),
			("cat/pkg[?a=]", False, False),
			("cat/pkg[?a?]", False, False),
			("cat/pkg[?a!]", False, False),
			("cat/pkg[?a-]", False, False),
			("sys-apps/portage[doc]:0", False, False),
			("*/*", False, False),
			("sys-apps/*", False, False),
			("*/portage", False, False),
			("*/**", True, False),
			("*/portage[use]", True, False),
			("cat/pkg[a()]", False, False),
			("cat/pkg[a(]", False, False),
			("cat/pkg[a)]", False, False),
			("cat/pkg[a(,b]", False, False),
			("cat/pkg[a),b]", False, False),
			("cat/pkg[a(*)]", False, False),
			("cat/pkg[a(*)]", True, False),
			("cat/pkg[a(+-)]", False, False),
			("cat/pkg[a()]", False, False),
			("cat/pkg[(+)a]", False, False),
			("cat/pkg[a=(+)]", False, False),
			("cat/pkg[!(+)a=]", False, False),
			("cat/pkg[!a=(+)]", False, False),
			("cat/pkg[a?(+)]", False, False),
			("cat/pkg[!a?(+)]", False, False),
			("cat/pkg[!(+)a?]", False, False),
			("cat/pkg[-(+)a]", False, False),
			("cat/pkg[a(+),-a]", False, False),
			("cat/pkg[a(-),-a]", False, False),
			("cat/pkg[-a,a(+)]", False, False),
			("cat/pkg[-a,a(-)]", False, False),
			("cat/pkg[-a(+),a(-)]", False, False),
			("cat/pkg[-a(-),a(+)]", False, False),
			("sys-apps/portage[doc]::repo_name", False, False),
			("sys-apps/portage:0[doc]::repo_name", False, False),
			("sys-apps/portage[doc]:0::repo_name", False, False),
			("=sys-apps/portage-2.1-r1:0::repo_name[doc,a=,!b=,c?,!d?,-e]", False, False),
			("=sys-apps/portage-2.1-r1*:0::repo_name[doc]", False, False),
			("sys-apps/portage:0::repo_name[doc]", False, False),
			("*/*::repo_name", True, False),
		)

		for atom, parts, allow_wildcard, allow_repo in tests:
			a = Atom(atom, allow_wildcard=allow_wildcard, allow_repo=allow_repo)
			op, cp, ver, slot, use, repo = parts
			self.assertEqual(op, a.operator,
				msg="Atom('%s').operator = %s == '%s'" % (atom, a.operator, op))
			self.assertEqual(cp, a.cp,
				msg="Atom('%s').cp = %s == '%s'" % (atom, a.cp, cp))
			if ver is not None:
				cpv = "%s-%s" % (cp, ver)
			else:
				cpv = cp
			self.assertEqual(cpv, a.cpv,
				msg="Atom('%s').cpv = %s == '%s'" % (atom, a.cpv, cpv))
			self.assertEqual(slot, a.slot,
				msg="Atom('%s').slot = %s == '%s'" % (atom, a.slot, slot))
			self.assertEqual(repo, a.repo,
				msg="Atom('%s').repo == %s == '%s'" % (atom, a.repo, repo))

			if a.use:
				returned_use = str(a.use)
			else:
				returned_use = None
			self.assertEqual(use, returned_use,
				msg="Atom('%s').use = %s == '%s'" % (atom, returned_use, use))

		for atom, allow_wildcard, allow_repo in tests_xfail:
			self.assertRaisesMsg(atom, (InvalidAtom, TypeError), Atom, atom,
				allow_wildcard=allow_wildcard, allow_repo=allow_repo)

	def testSlotAbiAtom(self):
		tests = (
			("virtual/ffmpeg:0/53", "5", {"slot": "0", "sub_slot": "53", "slot_operator": None}),
			("virtual/ffmpeg:0/53=", "5", {"slot": "0", "sub_slot": "53", "slot_operator": "="}),
			("virtual/ffmpeg:=", "5", {"slot": None, "sub_slot": None, "slot_operator": "="}),
			("virtual/ffmpeg:0=", "5", {"slot": "0", "sub_slot": None, "slot_operator": "="}),
			("virtual/ffmpeg:*", "5", {"slot": None, "sub_slot": None, "slot_operator": "*"}),
			("virtual/ffmpeg:0", "5", {"slot": "0", "sub_slot": None, "slot_operator": None}),
			("virtual/ffmpeg", "5", {"slot": None, "sub_slot": None, "slot_operator": None}),
		)

		for atom, eapi, parts in tests:
			a = Atom(atom, eapi=eapi)
			for k, v in parts.items():
				self.assertEqual(v, getattr(a, k),
					msg="Atom('%s').%s = %s == '%s'" %
					(atom, k, getattr(a, k), v))

	def test_intersects(self):
		test_cases = (
			("dev-libs/A", "dev-libs/A", True),
			("dev-libs/A", "dev-libs/B", False),
			("dev-libs/A", "sci-libs/A", False),
			("dev-libs/A[foo]", "sci-libs/A[bar]", False),
			("dev-libs/A[foo(+)]", "sci-libs/A[foo(-)]", False),
			("=dev-libs/A-1", "=dev-libs/A-1-r1", False),
			("~dev-libs/A-1", "=dev-libs/A-1", False),
			("=dev-libs/A-1:1", "=dev-libs/A-1", True),
			("=dev-libs/A-1:1", "=dev-libs/A-1:1", True),
			("=dev-libs/A-1:1", "=dev-libs/A-1:2", False),
		)

		for atom, other, expected_result in test_cases:
			self.assertEqual(Atom(atom).intersects(Atom(other)), expected_result,
				"%s and %s should intersect: %s" % (atom, other, expected_result))

	def test_violated_conditionals(self):
		test_cases = (
			("dev-libs/A", ["foo"], ["foo"], None, "dev-libs/A"),
			("dev-libs/A[foo]", [], ["foo"], None, "dev-libs/A[foo]"),
			("dev-libs/A[foo]", ["foo"], ["foo"], None, "dev-libs/A"),
			("dev-libs/A[foo]", [], ["foo"], [], "dev-libs/A[foo]"),
			("dev-libs/A[foo]", ["foo"], ["foo"], [], "dev-libs/A"),

			("dev-libs/A:0[foo]", ["foo"], ["foo"], [], "dev-libs/A:0"),

			("dev-libs/A[foo,-bar]", [], ["foo", "bar"], None, "dev-libs/A[foo]"),
			("dev-libs/A[-foo,bar]", [], ["foo", "bar"], None, "dev-libs/A[bar]"),

			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", [], ["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[a,!c=]"),

			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["a"], ["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["b"], ["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[a,b=,!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["c"], ["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[a]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["d"], ["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[a,!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["e"], ["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[a,!c=,!e?]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["f"], ["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[a,!c=,-f]"),

			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["a"], ["a", "b", "c", "d", "e", "f"], ["a"], "dev-libs/A[!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["b"], ["a", "b", "c", "d", "e", "f"], ["b"], "dev-libs/A[a,!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["c"], ["a", "b", "c", "d", "e", "f"], ["c"], "dev-libs/A[a,!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["d"], ["a", "b", "c", "d", "e", "f"], ["d"], "dev-libs/A[a,!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["e"], ["a", "b", "c", "d", "e", "f"], ["e"], "dev-libs/A[a,!c=]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["f"], ["a", "b", "c", "d", "e", "f"], ["f"], "dev-libs/A[a,!c=,-f]"),

			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", ["a"], ["a", "b", "c", "d", "e", "f"], ["a"], "dev-libs/A[!c(+)=]"),
			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]", ["b"], ["a", "b", "c", "d", "e", "f"], ["b"], "dev-libs/A[a(-),!c(-)=]"),
			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", ["c"], ["a", "b", "c", "d", "e", "f"], ["c"], "dev-libs/A[a(+),!c(+)=]"),
			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]", ["d"], ["a", "b", "c", "d", "e", "f"], ["d"], "dev-libs/A[a(-),!c(-)=]"),
			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", ["e"], ["a", "b", "c", "d", "e", "f"], ["e"], "dev-libs/A[a(+),!c(+)=]"),
			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]", ["f"], ["a", "b", "c", "d", "e", "f"], ["f"], "dev-libs/A[a(-),!c(-)=,-f(+)]"),

			("dev-libs/A[a(+),b(+)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", ["a"], ["a"], ["a"], "dev-libs/A[b(+)=,!e(+)?]"),
			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]", ["b"], ["b"], ["b"], "dev-libs/A[a(-),!c(-)=,-f(+)]"),
			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", ["c"], ["c"], ["c"], "dev-libs/A[!c(+)=,!e(+)?]"),
			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]", ["d"], ["d"], ["d"], "dev-libs/A[a(-),b(+)=,!c(-)=,-f(+)]"),
			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", ["e"], ["e"], ["e"], "dev-libs/A"),
			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]", ["f"], ["f"], ["f"], "dev-libs/A[a(-),b(+)=,!c(-)=,-f(+)]"),

			#Some more test cases to trigger all remaining code paths
			("dev-libs/B[x?]", [], ["x"], ["x"], "dev-libs/B[x?]"),
			("dev-libs/B[x(+)?]", [], [], ["x"], "dev-libs/B"),
			("dev-libs/B[x(-)?]", [], [], ["x"], "dev-libs/B[x(-)?]"),

			("dev-libs/C[x=]", [], ["x"], ["x"], "dev-libs/C[x=]"),
			("dev-libs/C[x(+)=]", [], [], ["x"], "dev-libs/C"),
			("dev-libs/C[x(-)=]", [], [], ["x"], "dev-libs/C[x(-)=]"),

			("dev-libs/D[!x=]", [], ["x"], ["x"], "dev-libs/D"),
			("dev-libs/D[!x(+)=]", [], [], ["x"], "dev-libs/D[!x(+)=]"),
			("dev-libs/D[!x(-)=]", [], [], ["x"], "dev-libs/D"),

			#Missing IUSE test cases
			("dev-libs/B[x]", [], [], [], "dev-libs/B[x]"),
			("dev-libs/B[-x]", [], [], [], "dev-libs/B[-x]"),
			("dev-libs/B[x?]", [], [], [], "dev-libs/B[x?]"),
			("dev-libs/B[x=]", [], [], [], "dev-libs/B[x=]"),
			("dev-libs/B[!x=]", [], [], ["x"], "dev-libs/B[!x=]"),
			("dev-libs/B[!x?]", [], [], ["x"], "dev-libs/B[!x?]"),
		)

		test_cases_xfail = (
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", [], ["a", "b", "c", "d", "e", "f"], None),
		)

		class use_flag_validator:
			def __init__(self, iuse):
				self.iuse = iuse

			def is_valid_flag(self, flag):
				return flag in iuse

		for atom, other_use, iuse, parent_use, expected_violated_atom in test_cases:
			a = Atom(atom)
			validator = use_flag_validator(iuse)
			violated_atom = a.violated_conditionals(other_use, validator.is_valid_flag, parent_use)
			if parent_use is None:
				fail_msg = "Atom: %s, other_use: %s, iuse: %s, parent_use: %s, got: %s, expected: %s" % \
					(atom, " ".join(other_use), " ".join(iuse), "None", str(violated_atom), expected_violated_atom)
			else:
				fail_msg = "Atom: %s, other_use: %s, iuse: %s, parent_use: %s, got: %s, expected: %s" % \
					(atom, " ".join(other_use), " ".join(iuse), " ".join(parent_use), str(violated_atom), expected_violated_atom)
			self.assertEqual(str(violated_atom), expected_violated_atom, fail_msg)

		for atom, other_use, iuse, parent_use in test_cases_xfail:
			a = Atom(atom)
			validator = use_flag_validator(iuse)
			self.assertRaisesMsg(atom, InvalidAtom,
				a.violated_conditionals, other_use, validator.is_valid_flag, parent_use)

	def test_evaluate_conditionals(self):
		test_cases = (
			("dev-libs/A[foo]", [], "dev-libs/A[foo]"),
			("dev-libs/A[foo]", ["foo"], "dev-libs/A[foo]"),

			("dev-libs/A:0[foo=]", ["foo"], "dev-libs/A:0[foo]"),

			("dev-libs/A[foo,-bar]", [], "dev-libs/A[foo,-bar]"),
			("dev-libs/A[-foo,bar]", [], "dev-libs/A[-foo,bar]"),

			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", [], "dev-libs/A[a,-b,c,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["a"], "dev-libs/A[a,-b,c,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["b"], "dev-libs/A[a,b,c,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["c"], "dev-libs/A[a,-b,-c,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["d"], "dev-libs/A[a,-b,c,d,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["e"], "dev-libs/A[a,-b,c,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["f"], "dev-libs/A[a,-b,c,-e,-f]"),
			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]", ["d"], "dev-libs/A[a(-),-b(+),c(-),d(+),-e(-),-f(+)]"),
			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]", ["f"], "dev-libs/A[a(+),-b(-),c(+),-e(+),-f(-)]"),
		)

		for atom, use, expected_atom in test_cases:
			a = Atom(atom)
			b = a.evaluate_conditionals(use)
			self.assertEqual(str(b), expected_atom)
			self.assertEqual(str(b.unevaluated_atom), atom)

	def test__eval_qa_conditionals(self):
		test_cases = (
			("dev-libs/A[foo]", [], [], "dev-libs/A[foo]"),
			("dev-libs/A[foo]", ["foo"], [], "dev-libs/A[foo]"),
			("dev-libs/A[foo]", [], ["foo"], "dev-libs/A[foo]"),

			("dev-libs/A:0[foo]", [], [], "dev-libs/A:0[foo]"),
			("dev-libs/A:0[foo]", ["foo"], [], "dev-libs/A:0[foo]"),
			("dev-libs/A:0[foo]", [], ["foo"], "dev-libs/A:0[foo]"),
			("dev-libs/A:0[foo=]", [], ["foo"], "dev-libs/A:0[foo]"),

			("dev-libs/A[foo,-bar]", ["foo"], ["bar"], "dev-libs/A[foo,-bar]"),
			("dev-libs/A[-foo,bar]", ["foo", "bar"], [], "dev-libs/A[-foo,bar]"),

			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["a", "b", "c"], [], "dev-libs/A[a,-b,c,d,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", [], ["a", "b", "c"], "dev-libs/A[a,b,-c,d,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", ["d", "e", "f"], [], "dev-libs/A[a,b,-b,c,-c,-e,-f]"),
			("dev-libs/A[a,b=,!c=,d?,!e?,-f]", [], ["d", "e", "f"], "dev-libs/A[a,b,-b,c,-c,d,-f]"),

			("dev-libs/A[a(-),b(+)=,!c(-)=,d(+)?,!e(-)?,-f(+)]",
				["a", "b", "c", "d", "e", "f"], [], "dev-libs/A[a(-),-b(+),c(-),-e(-),-f(+)]"),
			("dev-libs/A[a(+),b(-)=,!c(+)=,d(-)?,!e(+)?,-f(-)]",
				[], ["a", "b", "c", "d", "e", "f"], "dev-libs/A[a(+),b(-),-c(+),d(-),-f(-)]"),
		)

		for atom, use_mask, use_force, expected_atom in test_cases:
			a = Atom(atom)
			b = a._eval_qa_conditionals(use_mask, use_force)
			self.assertEqual(str(b), expected_atom)
			self.assertEqual(str(b.unevaluated_atom), atom)



# Copyright 2015-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

class SonameAtom:

	__slots__ = ("multilib_category", "soname", "_hash_key",
		"_hash_value")

	# Distiguishes package atoms from other atom types
	package = False

	def __init__(self, multilib_category, soname):
		object.__setattr__(self, "multilib_category", multilib_category)
		object.__setattr__(self, "soname", soname)
		object.__setattr__(self, "_hash_key",
			(multilib_category, soname))
		object.__setattr__(self, "_hash_value", hash(self._hash_key))

	def __setattr__(self, name, value):
		raise AttributeError("SonameAtom instances are immutable",
			self.__class__, name, value)

	def __getstate__(self):
		return dict((k, getattr(self, k)) for k in self.__slots__)

	def __setstate__(self, state):
		for k, v in state.items():
			object.__setattr__(self, k, v)

	def __hash__(self):
		return self._hash_value

	def __eq__(self, other):
		try:
			return self._hash_key == other._hash_key
		except AttributeError:
			return False

	def __ne__(self, other):
		try:
			return self._hash_key != other._hash_key
		except AttributeError:
			return True

	def __repr__(self):
		return "%s('%s', '%s')" % (
			self.__class__.__name__,
			self.multilib_category,
			self.soname
		)

	def __str__(self):
		return "%s: %s" % (self.multilib_category, self.soname)

	def match(self, pkg):
		"""
		Check if the given package instance matches this atom. Unbuilt
		ebuilds, which do not have soname metadata, will never match.

		@param pkg: a Package instance
		@type pkg: Package
		@return: True if this atom matches pkg, otherwise False
		@rtype: bool
		"""
		return pkg.provides is not None and self in pkg.provides



# Copyright 2016 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2
#
# This module provides an extended subset of the asyncio.futures.Futures
# interface.

__all__ = (
	'CancelledError',
	'ExtendedFuture',
	'InvalidStateError',
)

from portage.util.futures.futures import (Future, InvalidStateError,
	CancelledError)

# Create our one time settable unset constant
UNSET_CONST = Future()
UNSET_CONST.set_result(object())


class ExtendedFuture(Future):
	'''Extended Future class adding convienince get and set operations with
	default result capabilities for unset result().  It also adds pass
	capability for duplicate set_result() calls.
	'''

	def __init__(self, default_result=UNSET_CONST.result()):
		'''Class init

		@param default_result: Optional data type/value to return in the event
		                       of a result() call when result has not yet been
		                       set.
		'''
		self.default_result = default_result
		super(ExtendedFuture, self).__init__()
		self.set = self.set_result

	def set_result(self, data, ignore_InvalidState=False):
		'''Set the Future's result to the data, optionally don't raise
		an error for 'InvalidStateError' errors

		@param ignore_exception: Boolean
		'''
		if ignore_InvalidState:
			try:
				super(ExtendedFuture, self).set_result(data)
			except InvalidStateError:
				pass
		else:
			super(ExtendedFuture, self).set_result(data)

	def get(self, default=UNSET_CONST.result()):
		'''Convienience function to wrap result() but adds an optional
		default value to return rather than raise an InvalidStateError

		@param default: Optional override for the classwide default_result
		@returns: the result data or the default value, raisies an exception
		          if result is unset and no default is defined.
		'''
		if default is not UNSET_CONST.result():
			pass
		elif self.default_result is not UNSET_CONST.result():
			default = self.default_result
		if default is not UNSET_CONST.result():
			try:
				data = super(ExtendedFuture, self).result()
			except InvalidStateError:
				data = default
		else:
			data = super(ExtendedFuture, self).result()
		return data



# Copyright 2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (
	ResolverPlayground, ResolverPlaygroundTestCase)

class SonameSlotConflictReinstallTestCase(TestCase):

	def testSonameSlotConflictReinstall(self):

		binpkgs = {

			"app-misc/A-1" : {
				"PROVIDES": "x86_32: libA-1.so",
			},

			"app-misc/A-2" : {
				"PROVIDES": "x86_32: libA-2.so",
			},

			"app-misc/B-0" : {
				"DEPEND": "app-misc/A",
				"RDEPEND": "app-misc/A",
				"REQUIRES": "x86_32: libA-2.so",
			},

			"app-misc/C-0" : {
				"EAPI": "5",
				"DEPEND": "<app-misc/A-2",
				"RDEPEND": "<app-misc/A-2"
			},

			"app-misc/D-1" : {
				"PROVIDES": "x86_32: libD-1.so",
			},

			"app-misc/D-2" : {
				"PROVIDES": "x86_32: libD-2.so",
			},

			"app-misc/E-0" : {
				"DEPEND": "app-misc/D",
				"RDEPEND": "app-misc/D",
				"REQUIRES": "x86_32: libD-2.so",
			},

		}

		installed = {

			"app-misc/A-1" : {
				"PROVIDES": "x86_32: libA-1.so",
			},

			"app-misc/B-0" : {
				"DEPEND": "app-misc/A",
				"RDEPEND": "app-misc/A",
				"REQUIRES": "x86_32: libA-1.so",
			},

			"app-misc/C-0" : {
				"DEPEND": "<app-misc/A-2",
				"RDEPEND": "<app-misc/A-2"
			},

			"app-misc/D-1" : {
				"PROVIDES": "x86_32: libD-1.so",
			},

			"app-misc/E-0" : {
				"DEPEND": "app-misc/D",
				"RDEPEND": "app-misc/D",
				"REQUIRES": "x86_32: libD-1.so",
			},

		}

		world = ["app-misc/B", "app-misc/C", "app-misc/E"]

		test_cases = (

			# Test bug #439688, where a slot conflict prevents an
			# upgrade and we don't want to trigger unnecessary rebuilds.
			ResolverPlaygroundTestCase(
				["@world"],
				options = {
					"--deep": True,
					"--ignore-soname-deps": "n",
					"--update": True,
					"--usepkgonly": True,
					"--backtrack": 10,
				},
				success = True,
				mergelist = [
					"[binary]app-misc/D-2",
					"[binary]app-misc/E-0"
				]
			),

		)

		playground = ResolverPlayground(binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success,
					True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()

	def testSonameSlotConflictMassRebuild(self):
		"""
		Bug 486580
		Before this bug was fixed, emerge would backtrack for each
		package that needs a rebuild. This could cause it to hit the
		backtrack limit and not rebuild all needed packages.
		"""
		binpkgs = {

			"app-misc/A-1" : {
				"DEPEND": "app-misc/B",
				"RDEPEND": "app-misc/B",
				"REQUIRES": "x86_32: libB-2.so",
			},

			"app-misc/B-1" : {
				"SLOT": "1",
				"PROVIDES": "x86_32: libB-1.so",
			},

			"app-misc/B-2" : {
				"SLOT": "2",
				"PROVIDES": "x86_32: libB-2.so",
			},
		}

		installed = {
			"app-misc/B-1" : {
				"SLOT": "1",
				"PROVIDES": "x86_32: libB-1.so",
			},
		}

		expected_mergelist = [
			'[binary]app-misc/A-1',
			'[binary]app-misc/B-2'
		]

		for i in range(5):
			binpkgs["app-misc/C%sC-1" % i] = {
				"DEPEND": "app-misc/B",
				"RDEPEND": "app-misc/B",
				"REQUIRES": "x86_32: libB-2.so",
			}

			installed["app-misc/C%sC-1" % i] = {
				"DEPEND": "app-misc/B",
				"RDEPEND": "app-misc/B",
				"REQUIRES": "x86_32: libB-1.so",
			}
			for x in ("DEPEND", "RDEPEND"):
				binpkgs["app-misc/A-1"][x] += " app-misc/C%sC" % i

			expected_mergelist.append("[binary]app-misc/C%sC-1" % i)


		test_cases = (
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				ignore_mergelist_order=True,
				all_permutations=True,
				options = {
					"--backtrack": 3,
					"--deep": True,
					"--ignore-soname-deps": "n",
					"--update": True,
					"--usepkgonly": True,
				},
				success = True,
				mergelist = expected_mergelist),
		)

		world = []

		playground = ResolverPlayground(binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success,
					True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()

	def testSonameSlotConflictForgottenChild(self):
		"""
		Similar to testSonameSlotConflictMassRebuild above, but this
		time the rebuilds are scheduled, but the package causing the
		rebuild (the child) is not installed.
		"""
		binpkgs = {

			"app-misc/A-2" : {
				"DEPEND": "app-misc/B app-misc/C",
				"RDEPEND": "app-misc/B app-misc/C",
				"REQUIRES": "x86_32: libB-2.so",
			},

			"app-misc/B-2" : {
				"PROVIDES": "x86_32: libB-2.so",
				"SLOT": "2",
			},

			"app-misc/C-1": {
				"DEPEND": "app-misc/B",
				"RDEPEND": "app-misc/B",
				"REQUIRES": "x86_32: libB-2.so",
			},
		}

		installed = {
			"app-misc/A-1" : {
				"DEPEND": "app-misc/B app-misc/C",
				"RDEPEND": "app-misc/B app-misc/C",
				"REQUIRES": "x86_32: libB-1.so",
			},

			"app-misc/B-1" : {
				"PROVIDES": "x86_32: libB-1.so",
				"SLOT": "1",
			},

			"app-misc/C-1": {
				"DEPEND": "app-misc/B",
				"RDEPEND": "app-misc/B",
				"REQUIRES": "x86_32: libB-1.so",
			},
		}

		test_cases = (
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				options = {
					"--ignore-soname-deps": "n",
					"--usepkgonly": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/B-2',
					'[binary]app-misc/A-2',
				]
			),
			ResolverPlaygroundTestCase(
				["@world"],
				options = {
					"--ignore-soname-deps": "n",
					"--usepkgonly": True,
					"--update": True,
					"--deep": True,
				},
				success = True,
				mergelist = [
					'[binary]app-misc/B-2',
					'[binary]app-misc/C-1',
					'[binary]app-misc/A-2',
				]
			),
		)

		world = ['app-misc/A']

		playground = ResolverPlayground(binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()

	def testSonameSlotConflictMixedDependencies(self):
		"""
		Bug 487198
		For parents with mixed >= and < dependencies, we scheduled
		reinstalls for the >= atom, but in the end didn't install the
		child update because of the < atom.
		"""
		binpkgs = {
			"cat/slotted-lib-1" : {
				"PROVIDES": "x86_32: lib1.so",
				"SLOT": "1",
			},
			"cat/slotted-lib-2" : {
				"PROVIDES": "x86_32: lib2.so",
				"SLOT": "2",
			},
			"cat/slotted-lib-3" : {
				"PROVIDES": "x86_32: lib3.so",
				"SLOT": "3",
			},
			"cat/slotted-lib-4" : {
				"PROVIDES": "x86_32: lib4.so",
				"SLOT": "4",
			},
			"cat/slotted-lib-5" : {
				"PROVIDES": "x86_32: lib5.so",
				"SLOT": "5",
			},
			"cat/user-1" : {
				"DEPEND": ">=cat/slotted-lib-2 <cat/slotted-lib-4",
				"RDEPEND": ">=cat/slotted-lib-2 <cat/slotted-lib-4",
				"REQUIRES": "x86_32: lib3.so",
			},
		}

		installed = {
			"cat/slotted-lib-3" : {
				"PROVIDES": "x86_32: lib3.so",
				"SLOT": "3",
			},
			"cat/user-1" : {
				"DEPEND": ">=cat/slotted-lib-2 <cat/slotted-lib-4",
				"RDEPEND": ">=cat/slotted-lib-2 <cat/slotted-lib-4",
				"REQUIRES": "x86_32: lib3.so",
			},
		}

		test_cases = (
			ResolverPlaygroundTestCase(
				["cat/user"],
				options = {
					"--deep": True,
					"--ignore-soname-deps": "n",
					"--update": True,
					"--usepkgonly": True,
				},
				success = True,
				mergelist = []),
		)

		world = []

		playground = ResolverPlayground(binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success,
					True, test_case.fail_msg)
		finally:
			playground.debug = False
			playground.cleanup()



# Copyright 2014-2015 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

doc = """CVS plug-in module for portage.
Performs a cvs up on repositories."""
__doc__ = doc[:]

from portage.localization import _
from portage.sync.config_checks import CheckSyncConfig
from portage.util import writemsg_level


class CheckCVSConfig(CheckSyncConfig):

	def __init__(self, repo, logger):
		CheckSyncConfig.__init__(self, repo, logger)
		self.checks.append('check_cvs_repo')


	def check_cvs_repo(self):
		if self.repo.module_specific_options.get('sync-cvs-repo') is None:
			writemsg_level("!!! %s\n" %
				_("Repository '%s' has sync-type=cvs, but is missing sync-cvs-repo attribute")
				% self.repo.name, level=self.logger.ERROR, noiselevel=-1)


module_spec = {
	'name': 'cvs',
	'description': doc,
	'provides':{
		'cvs-module': {
			'name': "cvs",
			'sourcefile': "cvs",
			'class': "CVSSync",
			'description': doc,
			'functions': ['sync', 'new', 'exists'],
			'func_desc': {
				'sync': 'Performs a cvs up on the repository',
				'new': 'Creates the new repository at the specified location',
				'exists': 'Returns a boolean of whether the specified dir ' +
					'exists and is a valid CVS repository',
			},
			'validate_config': CheckCVSConfig,
			'module_specific_options': ("sync-cvs-repo",),
		}
	}
}



# Copyright 2010 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

class IpcCommand:

	__slots__ = ()

	def __call__(self, argv):
		raise NotImplementedError(self)



# Copyright 2007-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

__all__ = ["SETPREFIX", "get_boolean", "SetConfigError",
	"SetConfig", "load_default_config"]

import io
import logging
import sys
import portage
from portage import os
from portage import load_mod
from portage import _unicode_decode
from portage import _unicode_encode
from portage import _encodings
from portage.const import USER_CONFIG_PATH, GLOBAL_CONFIG_PATH
from portage.const import VCS_DIRS
from portage.const import _ENABLE_SET_CONFIG
from portage.exception import PackageSetNotFound
from portage.localization import _
from portage.util import writemsg_level
from portage.util.configparser import (SafeConfigParser,
	NoOptionError, ParsingError, read_configs)

SETPREFIX = "@"

def get_boolean(options, name, default):
	if not name in options:
		return default
	if options[name].lower() in ("1", "yes", "on", "true"):
		return True
	if options[name].lower() in ("0", "no", "off", "false"):
		return False
	raise SetConfigError(_("invalid value '%(value)s' for option '%(option)s'") % {"value": options[name], "option": name})

class SetConfigError(Exception):
	pass

class SetConfig:
	def __init__(self, paths, settings, trees):
		self._parser = SafeConfigParser(
			defaults={
				"EPREFIX" : settings["EPREFIX"],
				"EROOT" : settings["EROOT"],
				"PORTAGE_CONFIGROOT" : settings["PORTAGE_CONFIGROOT"],
				"ROOT" : settings["ROOT"],
			})

		if _ENABLE_SET_CONFIG:
			read_configs(self._parser, paths)
		else:
			self._create_default_config()

		self.errors = []
		self.psets = {}
		self.trees = trees
		self.settings = settings
		self._parsed = False
		self.active = []

	def _create_default_config(self):
		"""
		Create a default hardcoded set configuration for a portage version
		that does not support set configuration files. This is only used
		in the current branch of portage if _ENABLE_SET_CONFIG is False.
		Even if it's not used in this branch, keep it here in order to
		minimize the diff between branches.

			[world]
			class = portage.sets.base.DummyPackageSet
			packages = @selected @system

			[selected]
			class = portage.sets.files.WorldSelectedSet

			[system]
			class = portage.sets.profiles.PackagesSystemSet

		"""
		parser = self._parser

		parser.remove_section("world")
		parser.add_section("world")
		parser.set("world", "class", "portage.sets.base.DummyPackageSet")
		parser.set("world", "packages", "@profile @selected @system")

		parser.remove_section("profile")
		parser.add_section("profile")
		parser.set("profile", "class", "portage.sets.ProfilePackageSet.ProfilePackageSet")

		parser.remove_section("selected")
		parser.add_section("selected")
		parser.set("selected", "class", "portage.sets.files.WorldSelectedSet")

		parser.remove_section("selected-packages")
		parser.add_section("selected-packages")
		parser.set("selected-packages", "class", "portage.sets.files.WorldSelectedPackagesSet")

		parser.remove_section("selected-sets")
		parser.add_section("selected-sets")
		parser.set("selected-sets", "class", "portage.sets.files.WorldSelectedSetsSet")

		parser.remove_section("system")
		parser.add_section("system")
		parser.set("system", "class", "portage.sets.profiles.PackagesSystemSet")

		parser.remove_section("security")
		parser.add_section("security")
		parser.set("security", "class", "portage.sets.security.NewAffectedSet")

		parser.remove_section("usersets")
		parser.add_section("usersets")
		parser.set("usersets", "class", "portage.sets.files.StaticFileSet")
		parser.set("usersets", "multiset", "true")
		parser.set("usersets", "directory", "%(PORTAGE_CONFIGROOT)setc/portage/sets")
		parser.set("usersets", "world-candidate", "true")

		parser.remove_section("live-rebuild")
		parser.add_section("live-rebuild")
		parser.set("live-rebuild", "class", "portage.sets.dbapi.VariableSet")
		parser.set("live-rebuild", "variable", "PROPERTIES")
		parser.set("live-rebuild", "includes", "live")

		parser.remove_section("deprecated-live-rebuild")
		parser.add_section("deprecated-live-rebuild")
		parser.set("deprecated-live-rebuild", "class", "portage.sets.dbapi.VariableSet")
		parser.set("deprecated-live-rebuild", "variable", "INHERITED")
		parser.set("deprecated-live-rebuild", "includes", " ".join(sorted(portage.const.LIVE_ECLASSES)))

		parser.remove_section("module-rebuild")
		parser.add_section("module-rebuild")
		parser.set("module-rebuild", "class", "portage.sets.dbapi.OwnerSet")
		parser.set("module-rebuild", "files", "/lib/modules")

		parser.remove_section("preserved-rebuild")
		parser.add_section("preserved-rebuild")
		parser.set("preserved-rebuild", "class", "portage.sets.libs.PreservedLibraryConsumerSet")

		parser.remove_section("x11-module-rebuild")
		parser.add_section("x11-module-rebuild")
		parser.set("x11-module-rebuild", "class", "portage.sets.dbapi.OwnerSet")
		parser.set("x11-module-rebuild", "files", "/usr/lib*/xorg/modules")
		parser.set("x11-module-rebuild", "exclude-files", "/usr/bin/Xorg")

	def update(self, setname, options):
		parser = self._parser
		self.errors = []
		if not setname in self.psets:
			options["name"] = setname
			options["world-candidate"] = "False"

			# for the unlikely case that there is already a section with the requested setname
			import random
			while setname in parser.sections():
				setname = "%08d" % random.randint(0, 10**10)

			parser.add_section(setname)
			for k, v in options.items():
				parser.set(setname, k, v)
		else:
			section = self.psets[setname].creator
			if parser.has_option(section, "multiset") and \
				parser.getboolean(section, "multiset"):
				self.errors.append(_("Invalid request to reconfigure set '%(set)s' generated "
					"by multiset section '%(section)s'") % {"set": setname, "section": section})
				return
			for k, v in options.items():
				parser.set(section, k, v)
		self._parse(update=True)

	def _parse(self, update=False):
		if self._parsed and not update:
			return
		parser = self._parser
		for sname in parser.sections():
			# find classname for current section, default to file based sets
			if not parser.has_option(sname, "class"):
				classname = "portage._sets.files.StaticFileSet"
			else:
				classname = parser.get(sname, "class")

			if classname.startswith('portage.sets.'):
				# The module has been made private, but we still support
				# the previous namespace for sets.conf entries.
				classname = classname.replace('sets', '_sets', 1)

			# try to import the specified class
			try:
				setclass = load_mod(classname)
			except (ImportError, AttributeError):
				try:
					setclass = load_mod("portage._sets." + classname)
				except (ImportError, AttributeError):
					self.errors.append(_("Could not import '%(class)s' for section "
						"'%(section)s'") % {"class": classname, "section": sname})
					continue
			# prepare option dict for the current section
			optdict = {}
			for oname in parser.options(sname):
				optdict[oname] = parser.get(sname, oname)

			# create single or multiple instances of the given class depending on configuration
			if parser.has_option(sname, "multiset") and \
				parser.getboolean(sname, "multiset"):
				if hasattr(setclass, "multiBuilder"):
					newsets = {}
					try:
						newsets = setclass.multiBuilder(optdict, self.settings, self.trees)
					except SetConfigError as e:
						self.errors.append(_("Configuration error in section '%s': %s") % (sname, str(e)))
						continue
					for x in newsets:
						if x in self.psets and not update:
							self.errors.append(_("Redefinition of set '%s' (sections: '%s', '%s')") % (x, self.psets[x].creator, sname))
						newsets[x].creator = sname
						if parser.has_option(sname, "world-candidate") and \
							parser.getboolean(sname, "world-candidate"):
							newsets[x].world_candidate = True
					self.psets.update(newsets)
				else:
					self.errors.append(_("Section '%(section)s' is configured as multiset, but '%(class)s' "
						"doesn't support that configuration") % {"section": sname, "class": classname})
					continue
			else:
				try:
					setname = parser.get(sname, "name")
				except NoOptionError:
					setname = sname
				if setname in self.psets and not update:
					self.errors.append(_("Redefinition of set '%s' (sections: '%s', '%s')") % (setname, self.psets[setname].creator, sname))
				if hasattr(setclass, "singleBuilder"):
					try:
						self.psets[setname] = setclass.singleBuilder(optdict, self.settings, self.trees)
						self.psets[setname].creator = sname
						if parser.has_option(sname, "world-candidate") and \
							parser.getboolean(sname, "world-candidate"):
							self.psets[setname].world_candidate = True
					except SetConfigError as e:
						self.errors.append(_("Configuration error in section '%s': %s") % (sname, str(e)))
						continue
				else:
					self.errors.append(_("'%(class)s' does not support individual set creation, section '%(section)s' "
						"must be configured as multiset") % {"class": classname, "section": sname})
					continue
		self._parsed = True

	def getSets(self):
		self._parse()
		return self.psets.copy()

	def getSetAtoms(self, setname, ignorelist=None):
		"""
		This raises PackageSetNotFound if the give setname does not exist.
		"""
		self._parse()
		try:
			myset = self.psets[setname]
		except KeyError:
			raise PackageSetNotFound(setname)
		myatoms = myset.getAtoms()

		if ignorelist is None:
			ignorelist = set()

		ignorelist.add(setname)
		for n in myset.getNonAtoms():
			if n.startswith(SETPREFIX):
				s = n[len(SETPREFIX):]
				if s in self.psets:
					if s not in ignorelist:
						myatoms.update(self.getSetAtoms(s,
							ignorelist=ignorelist))
				else:
					raise PackageSetNotFound(s)

		return myatoms

def load_default_config(settings, trees):

	if not _ENABLE_SET_CONFIG:
		return SetConfig(None, settings, trees)

	global_config_path = GLOBAL_CONFIG_PATH
	if portage.const.EPREFIX:
		global_config_path = os.path.join(portage.const.EPREFIX,
			GLOBAL_CONFIG_PATH.lstrip(os.sep))
	vcs_dirs = [_unicode_encode(x, encoding=_encodings['fs']) for x in VCS_DIRS]
	def _getfiles():
		for path, dirs, files in os.walk(os.path.join(global_config_path, "sets")):
			for d in dirs:
				if d in vcs_dirs or d.startswith(b".") or d.endswith(b"~"):
					dirs.remove(d)
			for f in files:
				if not f.startswith(b".") and not f.endswith(b"~"):
					yield os.path.join(path, f)

		dbapi = trees["porttree"].dbapi
		for repo in dbapi.getRepositories():
			path = dbapi.getRepositoryPath(repo)
			yield os.path.join(path, "sets.conf")

		yield os.path.join(settings["PORTAGE_CONFIGROOT"],
			USER_CONFIG_PATH, "sets.conf")

	return SetConfig(_getfiles(), settings, trees)



# Copyright 2013-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import errno
import logging
import time

try:
	import threading
except ImportError:
	import dummy_threading as threading

import portage
from portage import os
from portage.util._async.TaskScheduler import TaskScheduler
from _emerge.CompositeTask import CompositeTask
from .FetchIterator import FetchIterator
from .DeletionIterator import DeletionIterator


class MirrorDistTask(CompositeTask):

	__slots__ = ('_config', '_fetch_iterator', '_term_rlock',
		'_term_callback_handle')

	def __init__(self, config):
		CompositeTask.__init__(self, scheduler=config.event_loop)
		self._config = config
		self._term_rlock = threading.RLock()
		self._term_callback_handle = None
		self._fetch_iterator = None

	def _start(self):
		self._fetch_iterator = FetchIterator(self._config)
		fetch = TaskScheduler(iter(self._fetch_iterator),
			max_jobs=self._config.options.jobs,
			max_load=self._config.options.load_average,
			event_loop=self._config.event_loop)
		self._start_task(fetch, self._fetch_exit)

	def _fetch_exit(self, fetch):

		self._assert_current(fetch)
		if self._was_cancelled():
			self._async_wait()
			return

		if self._config.options.delete:
			deletion = TaskScheduler(iter(DeletionIterator(self._config)),
				max_jobs=self._config.options.jobs,
				max_load=self._config.options.load_average,
				event_loop=self._config.event_loop)
			self._start_task(deletion, self._deletion_exit)
			return

		self._post_deletion()

	def _deletion_exit(self, deletion):

		self._assert_current(deletion)
		if self._was_cancelled():
			self._async_wait()
			return

		self._post_deletion()

	def _post_deletion(self):

		if self._config.options.recycle_db is not None:
			self._update_recycle_db()

		if self._config.options.scheduled_deletion_log is not None:
			self._scheduled_deletion_log()

		self._summary()

		self.returncode = os.EX_OK
		self._current_task = None
		self._async_wait()

	def _update_recycle_db(self):

		start_time = self._config.start_time
		recycle_dir = self._config.options.recycle_dir
		recycle_db = self._config.recycle_db
		r_deletion_delay = self._config.options.recycle_deletion_delay

		# Use a dict optimize access.
		recycle_db_cache = dict(recycle_db.items())

		for filename in os.listdir(recycle_dir):

			recycle_file = os.path.join(recycle_dir, filename)

			try:
				st = os.stat(recycle_file)
			except OSError as e:
				if e.errno not in (errno.ENOENT, errno.ESTALE):
					logging.error(("stat failed for '%s' in "
						"recycle: %s") % (filename, e))
				continue

			value = recycle_db_cache.pop(filename, None)
			if value is None:
				logging.debug(("add '%s' to "
					"recycle db") % filename)
				recycle_db[filename] = (st.st_size, start_time)
			else:
				r_size, r_time = value
				if int(r_size) != st.st_size:
					recycle_db[filename] = (st.st_size, start_time)
				elif r_time + r_deletion_delay < start_time:
					if self._config.options.dry_run:
						logging.info(("dry-run: delete '%s' from "
							"recycle") % filename)
						logging.info(("drop '%s' from "
							"recycle db") % filename)
					else:
						try:
							os.unlink(recycle_file)
						except OSError as e:
							if e.errno not in (errno.ENOENT, errno.ESTALE):
								logging.error(("delete '%s' from "
									"recycle failed: %s") % (filename, e))
						else:
							logging.debug(("delete '%s' from "
								"recycle") % filename)
							try:
								del recycle_db[filename]
							except KeyError:
								pass
							else:
								logging.debug(("drop '%s' from "
									"recycle db") % filename)

		# Existing files were popped from recycle_db_cache,
		# so any remaining entries are for files that no
		# longer exist.
		for filename in recycle_db_cache:
			try:
				del recycle_db[filename]
			except KeyError:
				pass
			else:
				logging.debug(("drop non-existent '%s' from "
					"recycle db") % filename)

	def _scheduled_deletion_log(self):

		start_time = self._config.start_time
		dry_run = self._config.options.dry_run
		deletion_delay = self._config.options.deletion_delay
		distfiles_db = self._config.distfiles_db

		date_map = {}
		for filename, timestamp in self._config.deletion_db.items():
			date = timestamp + deletion_delay
			if date < start_time:
				date = start_time
			date = time.strftime("%Y-%m-%d", time.gmtime(date))
			date_files = date_map.get(date)
			if date_files is None:
				date_files = []
				date_map[date] = date_files
			date_files.append(filename)

		if dry_run:
			logging.warning("dry-run: scheduled-deletions log "
				"will be summarized via logging.info")

		lines = []
		for date in sorted(date_map):
			date_files = date_map[date]
			if dry_run:
				logging.info(("dry-run: scheduled deletions for %s: %s files") %
					(date, len(date_files)))
			lines.append("%s\n" % date)
			for filename in date_files:
				cpv = "unknown"
				if distfiles_db is not None:
					cpv = distfiles_db.get(filename, cpv)
				lines.append("\t%s\t%s\n" % (filename, cpv))

		if not dry_run:
			portage.util.write_atomic(
				self._config.options.scheduled_deletion_log,
				"".join(lines))

	def _summary(self):
		elapsed_time = time.time() - self._config.start_time
		fail_count = len(self._config.file_failures)
		delete_count = self._config.delete_count
		scheduled_deletion_count = self._config.scheduled_deletion_count - delete_count
		added_file_count = self._config.added_file_count
		added_byte_count = self._config.added_byte_count

		logging.info("finished in %i seconds" % elapsed_time)
		logging.info("failed to fetch %i files" % fail_count)
		logging.info("deleted %i files" % delete_count)
		logging.info("deletion of %i files scheduled" %
			scheduled_deletion_count)
		logging.info("added %i files" % added_file_count)
		logging.info("added %i bytes total" % added_byte_count)

	def _cleanup(self):
		"""
		Cleanup any callbacks that have been registered with the global
		event loop.
		"""
		# The self._term_callback_handle attribute requires locking
		# since it's modified by the thread safe terminate method.
		with self._term_rlock:
			if self._term_callback_handle not in (None, False):
				self._term_callback_handle.cancel()
			# This prevents the terminate method from scheduling
			# any more callbacks (since _cleanup must eliminate all
			# callbacks in order to ensure complete cleanup).
			self._term_callback_handle = False

	def terminate(self):
		with self._term_rlock:
			if self._term_callback_handle is None:
				self._term_callback_handle = self.scheduler.call_soon_threadsafe(
					self._term_callback)

	def _term_callback(self):
		if self._fetch_iterator is not None:
			self._fetch_iterator.terminate()
		self.cancel()
		if self.returncode is None:
			# In this case, the exit callback for self._current_task will
			# trigger notification of exit listeners. Don't call _async_wait()
			# yet, since that could trigger event loop recursion if the
			# current (cancelled) task's exit callback does not set the
			# returncode first.
			pass
		else:
			self._async_wait()

	def _async_wait(self):
		"""
		Override _async_wait to call self._cleanup().
		"""
		self._cleanup()
		super(MirrorDistTask, self)._async_wait()



# Copyright 2014-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import datetime
import subprocess
import sys
import textwrap

import portage
from portage import os, shutil, _shell_quote
from portage import _unicode_decode
from portage.const import PORTAGE_PYM_PATH, TIMESTAMP_FORMAT
from portage.process import find_binary
from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import ResolverPlayground
from portage.util import ensure_dirs

class SyncLocalTestCase(TestCase):
	"""
	Test sync with rsync and git, using file:// sync-uri.
	"""

	def _must_skip(self):
		if find_binary("rsync") is None:
			return "rsync: command not found"
		if find_binary("git") is None:
			return "git: command not found"

	def testSyncLocal(self):
		debug = False

		skip_reason = self._must_skip()
		if skip_reason:
			self.portage_skip = skip_reason
			self.assertFalse(True, skip_reason)
			return

		repos_conf = textwrap.dedent("""
			[DEFAULT]
			%(default_keys)s
			[test_repo]
			location = %(EPREFIX)s/var/repositories/test_repo
			sync-type = %(sync-type)s
			sync-depth = %(sync-depth)s
			sync-uri = file://%(EPREFIX)s/var/repositories/test_repo_sync
			sync-rcu = %(sync-rcu)s
			sync-rcu-store-dir = %(EPREFIX)s/var/repositories/test_repo_rcu_storedir
			auto-sync = %(auto-sync)s
			%(repo_extra_keys)s
		""")

		profile = {
			"eapi": ("5",),
			"package.use.stable.mask": ("dev-libs/A flag",)
		}

		ebuilds = {
			"dev-libs/A-0": {},
			"sys-apps/portage-3.0": {"IUSE": "+python_targets_python3_8"},
		}

		installed = {
			"sys-apps/portage-2.3.99": {
				"EAPI": "7",
				"IUSE": "+python_targets_python3_8",
				"USE": "python_targets_python3_8",
			},
		}

		user_config = {
			'make.conf': ('FEATURES="metadata-transfer"',)
		}

		playground = ResolverPlayground(ebuilds=ebuilds, installed=installed,
			profile=profile, user_config=user_config, debug=debug)
		settings = playground.settings
		eprefix = settings["EPREFIX"]
		eroot = settings["EROOT"]
		homedir = os.path.join(eroot, "home")
		distdir = os.path.join(eprefix, "distdir")
		repo = settings.repositories["test_repo"]
		metadata_dir = os.path.join(repo.location, "metadata")
		rcu_store_dir = os.path.join(eprefix, 'var/repositories/test_repo_rcu_storedir')

		cmds = {}
		for cmd in ("emerge", "emaint"):
			for bindir in (self.bindir, self.sbindir):
				path = os.path.join(bindir, cmd)
				if os.path.exists(path):
					cmds[cmd] =  (portage._python_interpreter,
						"-b", "-Wd", path)
					break
			else:
				raise AssertionError('%s binary not found in %s or %s' %
					(cmd, self.bindir, self.sbindir))

		git_binary = find_binary("git")
		git_cmd = (git_binary,)

		hg_binary = find_binary("hg")
		hg_cmd = (hg_binary,)

		committer_name = "Gentoo Dev"
		committer_email = "gentoo-dev@gentoo.org"

		def repos_set_conf(sync_type, dflt_keys=None, xtra_keys=None,
			auto_sync="yes", sync_rcu=False, sync_depth=None):
			env["PORTAGE_REPOSITORIES"] = repos_conf % {\
				"EPREFIX": eprefix, "sync-type": sync_type,
				"sync-depth": 0 if sync_depth is None else sync_depth,
				"sync-rcu": "yes" if sync_rcu else "no",
				"auto-sync": auto_sync,
				"default_keys": "" if dflt_keys is None else dflt_keys,
				"repo_extra_keys": "" if xtra_keys is None else xtra_keys}

		def alter_ebuild():
			with open(os.path.join(repo.location + "_sync",
				"dev-libs", "A", "A-0.ebuild"), "a") as f:
				f.write("\n")
			bump_timestamp()

		def bump_timestamp():
			bump_timestamp.timestamp += datetime.timedelta(seconds=1)
			with open(os.path.join(repo.location + '_sync', 'metadata', 'timestamp.chk'), 'w') as f:
				f.write(bump_timestamp.timestamp.strftime('%s\n' % TIMESTAMP_FORMAT,))

		bump_timestamp.timestamp = datetime.datetime.utcnow()

		bump_timestamp_cmds = (
			(homedir, bump_timestamp),
		)

		sync_cmds = (
			(homedir, cmds["emerge"] + ("--sync",)),
			(homedir, lambda: self.assertTrue(os.path.exists(
				os.path.join(repo.location, "dev-libs", "A")
				), "dev-libs/A expected, but missing")),
			(homedir, cmds["emaint"] + ("sync", "-A")),
		)

		sync_cmds_auto_sync = (
			(homedir, lambda: repos_set_conf("rsync", auto_sync="no")),
			(homedir, cmds["emerge"] + ("--sync",)),
			(homedir, lambda: self.assertFalse(os.path.exists(
				os.path.join(repo.location, "dev-libs", "A")
				), "dev-libs/A found, expected missing")),
			(homedir, lambda: repos_set_conf("rsync", auto_sync="yes")),
		)

		rename_repo = (
			(homedir, lambda: os.rename(repo.location,
				repo.location + "_sync")),
		)

		rsync_opts_repos = (
			(homedir, alter_ebuild),
			(homedir, lambda: repos_set_conf("rsync", None,
				"sync-rsync-extra-opts = --backup --backup-dir=%s" %
				_shell_quote(repo.location + "_back"))),
			(homedir, cmds['emerge'] + ("--sync",)),
			(homedir, lambda: self.assertTrue(os.path.exists(
				repo.location + "_back"))),
			(homedir, lambda: shutil.rmtree(repo.location + "_back")),
			(homedir, lambda: repos_set_conf("rsync")),
		)

		rsync_opts_repos_default = (
			(homedir, alter_ebuild),
			(homedir, lambda: repos_set_conf("rsync",
					"sync-rsync-extra-opts = --backup --backup-dir=%s" %
					_shell_quote(repo.location+"_back"))),
			(homedir, cmds['emerge'] + ("--sync",)),
			(homedir, lambda: self.assertTrue(os.path.exists(repo.location + "_back"))),
			(homedir, lambda: shutil.rmtree(repo.location + "_back")),
			(homedir, lambda: repos_set_conf("rsync")),
		)

		rsync_opts_repos_default_ovr = (
			(homedir, alter_ebuild),
			(homedir, lambda: repos_set_conf("rsync",
				"sync-rsync-extra-opts = --backup --backup-dir=%s" %
				_shell_quote(repo.location + "_back_nowhere"),
				"sync-rsync-extra-opts = --backup --backup-dir=%s" %
				_shell_quote(repo.location + "_back"))),
			(homedir, cmds['emerge'] + ("--sync",)),
			(homedir, lambda: self.assertTrue(os.path.exists(repo.location + "_back"))),
			(homedir, lambda: shutil.rmtree(repo.location + "_back")),
			(homedir, lambda: repos_set_conf("rsync")),
		)

		rsync_opts_repos_default_cancel = (
			(homedir, alter_ebuild),
			(homedir, lambda: repos_set_conf("rsync",
				"sync-rsync-extra-opts = --backup --backup-dir=%s" %
				_shell_quote(repo.location + "_back_nowhere"),
				"sync-rsync-extra-opts = ")),
			(homedir, cmds['emerge'] + ("--sync",)),
			(homedir, lambda: self.assertFalse(os.path.exists(repo.location + "_back"))),
			(homedir, lambda: repos_set_conf("rsync")),
		)

		delete_repo_location = (
			(homedir, lambda: shutil.rmtree(repo.user_location)),
			(homedir, lambda: os.mkdir(repo.user_location)),
		)

		delete_rcu_store_dir = (
			(homedir, lambda: shutil.rmtree(rcu_store_dir)),
		)

		revert_rcu_layout = (
			(homedir, lambda: os.rename(repo.user_location, repo.user_location + '.bak')),
			(homedir, lambda: os.rename(os.path.realpath(repo.user_location + '.bak'), repo.user_location)),
			(homedir, lambda: os.unlink(repo.user_location + '.bak')),
			(homedir, lambda: shutil.rmtree(repo.user_location + '_rcu_storedir')),
		)

		upstream_git_commit = (
			(
				repo.location + "_sync",
				git_cmd + ('commit', '--allow-empty', '-m', 'test empty commit'),
			),
			(
				repo.location + "_sync",
				git_cmd + ('commit', '--allow-empty', '-m', 'test empty commit 2'),
			),
		)

		delete_sync_repo = (
			(homedir, lambda: shutil.rmtree(
				repo.location + "_sync")),
		)

		git_repo_create = (
			(repo.location, git_cmd +
				("config", "--global", "user.name", committer_name,)),
			(repo.location, git_cmd +
				("config", "--global", "user.email", committer_email,)),
			(repo.location, git_cmd + ("init-db",)),
			(repo.location, git_cmd + ("add", ".")),
			(repo.location, git_cmd +
				("commit", "-a", "-m", "add whole repo")),
		)

		sync_type_git = (
			(homedir, lambda: repos_set_conf("git")),
		)

		sync_type_git_shallow = (
			(homedir, lambda: repos_set_conf("git", sync_depth=1)),
		)

		sync_rsync_rcu = (
			(homedir, lambda: repos_set_conf("rsync", sync_rcu=True)),
		)

		delete_git_dir = (
			(homedir, lambda: shutil.rmtree(
				os.path.join(repo.location, ".git"))),
		)

		def hg_init_global_config():
			with open(os.path.join(homedir, ".hgrc"), "wt") as f:
				f.write(
					"[ui]\nusername = {} <{}>\n".format(committer_name, committer_email)
				)

		hg_repo_create = (
			(repo.location, hg_init_global_config),
			(repo.location, hg_cmd + ("init",)),
			(repo.location, hg_cmd + ("add", ".")),
			(repo.location, hg_cmd + ("commit", "-A", "-m", "add whole repo")),
		)

		sync_type_mercurial = ((homedir, lambda: repos_set_conf("mercurial")),)

		def append_newline(path):
			with open(path, "at") as f:
				f.write("\n")

		upstream_hg_commit = (
			(
				repo.location + "_sync",
				lambda: append_newline(
					os.path.join(repo.location + "_sync", "metadata/layout.conf")
				),
			),
			(
				repo.location + "_sync",
				hg_cmd + ("commit", "metadata/layout.conf", "-m", "test empty commit"),
			),
			(
				repo.location + "_sync",
				lambda: append_newline(
					os.path.join(repo.location + "_sync", "metadata/layout.conf")
				),
			),
			(
				repo.location + "_sync",
				hg_cmd
				+ ("commit", "metadata/layout.conf", "-m", "test empty commit 2"),
			),
		)

		if hg_binary is None:
			mercurial_tests = ()
		else:
			mercurial_tests = (
				delete_sync_repo
				+ delete_git_dir
				+ hg_repo_create
				+ sync_type_mercurial
				+ rename_repo
				+ sync_cmds
				+ upstream_hg_commit
				+ sync_cmds
			)

		pythonpath =  os.environ.get("PYTHONPATH")
		if pythonpath is not None and not pythonpath.strip():
			pythonpath = None
		if pythonpath is not None and \
			pythonpath.split(":")[0] == PORTAGE_PYM_PATH:
			pass
		else:
			if pythonpath is None:
				pythonpath = ""
			else:
				pythonpath = ":" + pythonpath
			pythonpath = PORTAGE_PYM_PATH + pythonpath

		env = {
			"PORTAGE_OVERRIDE_EPREFIX" : eprefix,
			"DISTDIR" : distdir,
			"GENTOO_COMMITTER_NAME" : committer_name,
			"GENTOO_COMMITTER_EMAIL" : committer_email,
			"HOME" : homedir,
			"PATH" : os.environ["PATH"],
			"PORTAGE_GRPNAME" : os.environ["PORTAGE_GRPNAME"],
			"PORTAGE_USERNAME" : os.environ["PORTAGE_USERNAME"],
			"PYTHONDONTWRITEBYTECODE" : os.environ.get("PYTHONDONTWRITEBYTECODE", ""),
			"PYTHONPATH" : pythonpath,
		}
		repos_set_conf("rsync")

		if os.environ.get("SANDBOX_ON") == "1":
			# avoid problems from nested sandbox instances
			env["FEATURES"] = "-sandbox -usersandbox"

		dirs = [homedir, metadata_dir]
		try:
			for d in dirs:
				ensure_dirs(d)

			timestamp_path = os.path.join(metadata_dir, 'timestamp.chk')
			with open(timestamp_path, 'w') as f:
				f.write(bump_timestamp.timestamp.strftime('%s\n' % TIMESTAMP_FORMAT,))

			if debug:
				# The subprocess inherits both stdout and stderr, for
				# debugging purposes.
				stdout = None
			else:
				# The subprocess inherits stderr so that any warnings
				# triggered by python -Wd will be visible.
				stdout = subprocess.PIPE

			for cwd, cmd in rename_repo + sync_cmds_auto_sync + sync_cmds + \
				rsync_opts_repos + rsync_opts_repos_default + \
				rsync_opts_repos_default_ovr + rsync_opts_repos_default_cancel + \
				bump_timestamp_cmds + sync_rsync_rcu + sync_cmds + delete_rcu_store_dir + \
				sync_cmds + revert_rcu_layout + \
				delete_repo_location + sync_cmds + sync_cmds + \
				bump_timestamp_cmds + sync_cmds + revert_rcu_layout + \
				delete_sync_repo + git_repo_create + sync_type_git + \
				rename_repo + sync_cmds + upstream_git_commit + sync_cmds + \
				sync_type_git_shallow + upstream_git_commit + sync_cmds + mercurial_tests:

				if hasattr(cmd, '__call__'):
					cmd()
					continue

				abs_cwd = os.path.join(repo.location, cwd)
				proc = subprocess.Popen(cmd,
					cwd=abs_cwd, env=env, stdout=stdout)

				if debug:
					proc.wait()
				else:
					output = proc.stdout.readlines()
					proc.wait()
					proc.stdout.close()
					if proc.returncode != os.EX_OK:
						for line in output:
							sys.stderr.write(_unicode_decode(line))

				self.assertEqual(os.EX_OK, proc.returncode,
					"%s failed in %s" % (cmd, cwd,))


		finally:
			playground.cleanup()



# Copyright 2012-2020 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import re
import subprocess
import sys

from portage import _unicode_encode, _unicode_decode
from portage.util import writemsg
from portage.util.configparser import (RawConfigParser,	read_configs)

def parse_desktop_entry(path):
	"""
	Parse the given file with RawConfigParser and return the
	result. This may raise an IOError from io.open(), or a
	ParsingError from RawConfigParser.
	"""
	parser = RawConfigParser()

	read_configs(parser, [path])

	return parser

_trivial_warnings = re.compile(r' looks '
	# >=desktop-file-utils-0.25
	r'(?:the same as that of key|'

	# <desktop-file-utils-0.25
	r'redundant with value) ')

_ignored_errors = (
		# Ignore error for emacs.desktop:
		# https://bugs.freedesktop.org/show_bug.cgi?id=35844#c6
		'error: (will be fatal in the future): value "TextEditor" in key "Categories" in group "Desktop Entry" requires another category to be present among the following categories: Utility',
		'warning: key "Encoding" in group "Desktop Entry" is deprecated'
)

_ShowIn_exemptions = (
	# See bug #480586.
	'contains an unregistered value "Pantheon"',
)

def validate_desktop_entry(path):
	args = ["desktop-file-validate", path]

	args = [_unicode_encode(x, errors='strict') for x in args]
	proc = subprocess.Popen(args,
		stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
	output_lines = _unicode_decode(proc.communicate()[0]).splitlines()
	proc.wait()

	if output_lines:
		filtered_output = []
		for line in output_lines:
			msg = line[len(path)+2:]
			# "hint:" output is new in desktop-file-utils-0.21
			if msg.startswith('hint: ') or msg in _ignored_errors:
				continue
			if 'for key "NotShowIn" in group "Desktop Entry"' in msg or \
				'for key "OnlyShowIn" in group "Desktop Entry"' in msg:
				exempt = False
				for s in _ShowIn_exemptions:
					if s in msg:
						exempt = True
						break
				if exempt:
					continue
			filtered_output.append(line)
		output_lines = filtered_output

	if output_lines:
		output_lines = [line for line in output_lines
			if _trivial_warnings.search(line) is None]

	return output_lines

if __name__ == "__main__":
	for arg in sys.argv[1:]:
		for line in validate_desktop_entry(arg):
			writemsg(line + "\n", noiselevel=-1)



# Copyright 2014 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class AutounmaskUseBreakageTestCase(TestCase):

	def testAutounmaskUseBreakage(self):

		ebuilds = {

			"app-misc/A-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/D[-foo]",
			},

			"app-misc/B-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/D[foo]"
			},

			"app-misc/C-0" : {
				"EAPI": "5",
				"RDEPEND": ">=app-misc/D-1"
			},

			"app-misc/D-0" : {
				"EAPI": "5",
				"IUSE": "foo"
			},

			"app-misc/D-1" : {
				"EAPI": "5",
				"IUSE": "bar"
			},

		}

		test_cases = (

			# Bug 510270
			# _solve_non_slot_operator_slot_conflicts throws
			# IndexError: tuple index out of range
			# due to autounmask USE breakage.
			ResolverPlaygroundTestCase(
				["app-misc/C", "app-misc/B", "app-misc/A"],
				options={"--autounmask-backtrack": "y"},
				all_permutations = True,
				success = False,
				ambiguous_slot_collision_solutions = True,
				slot_collision_solutions = [None, []]
			),

			# With --autounmask-backtrack=y:
			#emerge: there are no ebuilds built with USE flags to satisfy "app-misc/D[foo]".
			#!!! One of the following packages is required to complete your request:
			#- app-misc/D-0::test_repo (Change USE: +foo)
			#(dependency required by "app-misc/B-0::test_repo" [ebuild])
			#(dependency required by "app-misc/B" [argument])

			# Without --autounmask-backtrack=y:
			#[ebuild  N     ] app-misc/D-0  USE="foo"
			#[ebuild  N     ] app-misc/D-1  USE="-bar"
			#[ebuild  N     ] app-misc/C-0
			#[ebuild  N     ] app-misc/B-0
			#[ebuild  N     ] app-misc/A-0
			#
			#!!! Multiple package instances within a single package slot have been pulled
			#!!! into the dependency graph, resulting in a slot conflict:
			#
			#app-misc/D:0
			#
			#  (app-misc/D-0:0/0::test_repo, ebuild scheduled for merge) pulled in by
			#    app-misc/D[-foo] required by (app-misc/A-0:0/0::test_repo, ebuild scheduled for merge)
			#               ^^^^
			#    app-misc/D[foo] required by (app-misc/B-0:0/0::test_repo, ebuild scheduled for merge)
			#               ^^^
			#
			#  (app-misc/D-1:0/0::test_repo, ebuild scheduled for merge) pulled in by
			#    >=app-misc/D-1 required by (app-misc/C-0:0/0::test_repo, ebuild scheduled for merge)
			#    ^^           ^
			#
			#The following USE changes are necessary to proceed:
			# (see "package.use" in the portage(5) man page for more details)
			## required by app-misc/B-0::test_repo
			## required by app-misc/B (argument)
			#=app-misc/D-0 foo

			# NOTE: The --autounmask-backtrack=n output is preferable here,
			# because it highlights the unsolvable dependency conflict.
			# It would be better if it eliminated the autounmask suggestion,
			# since that suggestion won't solve the conflict.
		)

		playground = ResolverPlayground(ebuilds=ebuilds, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



# Copyright 2010-2012 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.dep import cpvequal
from portage.exception import PortageException

class TestStandalone(TestCase):
	""" Test some small functions portage.dep
	"""

	def testCPVequal(self):

		test_cases = (
			("sys-apps/portage-2.1", "sys-apps/portage-2.1", True),
			("sys-apps/portage-2.1", "sys-apps/portage-2.0", False),
			("sys-apps/portage-2.1", "sys-apps/portage-2.1-r1", False),
			("sys-apps/portage-2.1-r1", "sys-apps/portage-2.1", False),
			("sys-apps/portage-2.1_alpha3", "sys-apps/portage-2.1", False),
			("sys-apps/portage-2.1_alpha3_p6", "sys-apps/portage-2.1_alpha3", False),
			("sys-apps/portage-2.1_alpha3", "sys-apps/portage-2.1", False),
			("sys-apps/portage-2.1", "sys-apps/X-2.1", False),
			("sys-apps/portage-2.1", "portage-2.1", False),
		)

		test_cases_xfail = (
			("sys-apps/portage", "sys-apps/portage"),
			("sys-apps/portage-2.1-6", "sys-apps/portage-2.1-6"),
		)

		for cpv1, cpv2, expected_result in test_cases:
			self.assertEqual(cpvequal(cpv1, cpv2), expected_result,
				"cpvequal('%s', '%s') != %s" % (cpv1, cpv2, expected_result))

		for cpv1, cpv2 in test_cases_xfail:
			self.assertRaisesMsg("cpvequal(%s, %s)" % (cpv1, cpv2),
				PortageException, cpvequal, cpv1, cpv2)



# testStaticFileSet.py -- Portage Unit Testing Functionality
# Copyright 2007 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

import tempfile

from portage import os
from portage.tests import TestCase, test_cps
from portage._sets.files import StaticFileSet

class StaticFileSetTestCase(TestCase):
	"""Simple Test Case for StaticFileSet"""

	def setUp(self):
		fd, self.testfile = tempfile.mkstemp(suffix=".testdata", prefix=self.__class__.__name__, text=True)
		f = os.fdopen(fd, 'w')
		f.write("\n".join(test_cps))
		f.close()

	def tearDown(self):
		os.unlink(self.testfile)

	def testSampleStaticFileSet(self):
		s = StaticFileSet(self.testfile)
		s.load()
		self.assertEqual(set(test_cps), s.getAtoms())




'''WebRsync module for portage'''

import io
import logging

import portage
from portage import os
from portage.util import writemsg_level
from portage.util.futures import asyncio
from portage.output import create_color_func
good = create_color_func("GOOD")
bad = create_color_func("BAD")
warn = create_color_func("WARN")
from portage.sync.syncbase import SyncBase

try:
	from gemato.exceptions import GematoException
	import gemato.openpgp
except ImportError:
	gemato = None


class WebRsync(SyncBase):
	'''WebRSync sync class'''

	short_desc = "Perform sync operations on webrsync based repositories"

	@staticmethod
	def name():
		return "WebRSync"


	def __init__(self):
		SyncBase.__init__(self, 'emerge-webrsync', '>=sys-apps/portage-2.3')

	@property
	def has_bin(self):
		if (self._bin_command != 'emerge-delta-webrsync' and
			self.repo.module_specific_options.get(
			'sync-webrsync-delta', 'false').lower() in ('true', 'yes')):
			self._bin_command = 'emerge-delta-webrsync'
			self.bin_command = portage.process.find_binary(self._bin_command)
			self.bin_pkg = '>=app-portage/emerge-delta-webrsync-3.7.5'

		return super(WebRsync, self).has_bin

	def sync(self, **kwargs):
		'''Sync the repository'''
		if kwargs:
			self._kwargs(kwargs)

		if not self.has_bin:
			return (1, False)

		# filter these out to prevent gpg errors
		for var in ['uid', 'gid', 'groups']:
			self.spawn_kwargs.pop(var, None)

		verbose = '--verbose' in self.options['emerge_config'].opts
		quiet = '--quiet' in self.options['emerge_config'].opts
		openpgp_env = None
		try:
			if self.repo.module_specific_options.get(
				'sync-webrsync-verify-signature', 'false').lower() in ('true', 'yes'):

				if not self.repo.sync_openpgp_key_path:
					writemsg_level("!!! sync-openpgp-key-path is not set\n",
						level=logging.ERROR, noiselevel=-1)
					return (1, False)

				if not os.path.isfile(self.repo.sync_openpgp_key_path):
					writemsg_level("!!! sync-openpgp-key-path file not found: %s\n" %
						self.repo.sync_openpgp_key_path, level=logging.ERROR, noiselevel=-1)
					return (1, False)

				if gemato is None:
					writemsg_level("!!! Verifying against specified key requires gemato-14.5+ installed\n",
						level=logging.ERROR, noiselevel=-1)
					return (1, False)

				openpgp_env = self._get_openpgp_env(self.repo.sync_openpgp_key_path)

				out = portage.output.EOutput(quiet=quiet)
				try:
					out.einfo('Using keys from %s' % (self.repo.sync_openpgp_key_path,))
					with io.open(self.repo.sync_openpgp_key_path, 'rb') as f:
						openpgp_env.import_key(f)
					self._refresh_keys(openpgp_env)
					self.spawn_kwargs["env"]["PORTAGE_GPG_DIR"] = openpgp_env.home
					self.spawn_kwargs["env"]["PORTAGE_TEMP_GPG_DIR"] = openpgp_env.home
				except (GematoException, asyncio.TimeoutError) as e:
					writemsg_level("!!! Verification impossible due to keyring problem:\n%s\n"
							% (e,),
							level=logging.ERROR, noiselevel=-1)
					return (1, False)

			webrsync_cmd = [self.bin_command]
			if verbose:
				webrsync_cmd.append('-v')
			elif quiet:
				webrsync_cmd.append('-q')

			if self.repo.module_specific_options.get(
				'sync-webrsync-keep-snapshots', 'false').lower() in ('true', 'yes'):
				webrsync_cmd.append('-k')

			exitcode = portage.process.spawn(webrsync_cmd, **self.spawn_kwargs)
			if exitcode != os.EX_OK:
				msg = "!!! emerge-webrsync error in %s" % self.repo.location
				self.logger(self.xterm_titles, msg)
				writemsg_level(msg + "\n", level=logging.ERROR, noiselevel=-1)
				return (exitcode, False)
			return (exitcode, True)
		finally:
			if openpgp_env is not None:
				openpgp_env.close()


class PyWebRsync(SyncBase):
	'''WebRSync sync class'''

	short_desc = "Perform sync operations on webrsync based repositories"

	@staticmethod
	def name():
		return "WebRSync"


	def __init__(self):
		SyncBase.__init__(self, None, '>=sys-apps/portage-2.3')


	def sync(self, **kwargs):
		'''Sync the repository'''
		pass



# Copyright 1999-2011 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from _emerge.DependencyArg import DependencyArg
from _emerge.Package import Package
import portage
from portage._sets.base import InternalPackageSet
from portage.dep import _repo_separator

class PackageArg(DependencyArg):
	def __init__(self, package=None, **kwargs):
		DependencyArg.__init__(self, **kwargs)
		self.package = package
		atom = "=" + package.cpv
		if package.repo != Package.UNKNOWN_REPO:
			atom += _repo_separator + package.repo
		self.atom = portage.dep.Atom(atom, allow_repo=True)
		self.pset = InternalPackageSet(initial_atoms=(self.atom,),
			allow_repo=True)



# Copyright 2008-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import fcntl
import errno
import gzip

import portage
from portage import os, _encodings, _unicode_encode
from portage.util.futures import asyncio
from portage.util.futures._asyncio.streams import _writer
from portage.util.futures.unix_events import _set_nonblocking
from _emerge.AbstractPollTask import AbstractPollTask

class PipeLogger(AbstractPollTask):

	"""
	This can be used for logging output of a child process,
	optionally outputing to log_file_path and/or stdout_fd.  It can
	also monitor for EOF on input_fd, which may be used to detect
	termination of a child process. If log_file_path ends with
	'.gz' then the log file is written with compression.
	"""

	__slots__ = ("input_fd", "log_file_path", "stdout_fd") + \
		("_io_loop_task", "_log_file", "_log_file_nb", "_log_file_real")

	def _start(self):

		log_file_path = self.log_file_path
		if hasattr(log_file_path, 'write'):
			self._log_file_nb = True
			self._log_file = log_file_path
			_set_nonblocking(self._log_file.fileno())
		elif log_file_path is not None:
			try:
				self._log_file = open(
					_unicode_encode(
						log_file_path, encoding=_encodings["fs"], errors="strict"
					),
					mode="ab",
				)

				if log_file_path.endswith(".gz"):
					self._log_file_real = self._log_file
					self._log_file = gzip.GzipFile(
						filename="", mode="ab", fileobj=self._log_file
					)

				portage.util.apply_secpass_permissions(
					log_file_path,
					uid=portage.portage_uid,
					gid=portage.portage_gid,
					mode=0o660,
				)
			except FileNotFoundError:
				if self._was_cancelled():
					self._async_wait()
					return
				raise

		if isinstance(self.input_fd, int):
			self.input_fd = os.fdopen(self.input_fd, 'rb', 0)

		fd = self.input_fd.fileno()

		fcntl.fcntl(fd, fcntl.F_SETFL,
			fcntl.fcntl(fd, fcntl.F_GETFL) | os.O_NONBLOCK)

		self._io_loop_task = asyncio.ensure_future(self._io_loop(self.input_fd), loop=self.scheduler)
		self._io_loop_task.add_done_callback(self._io_loop_done)
		self._registered = True

	def _cancel(self):
		self._unregister()
		if self.returncode is None:
			self.returncode = self._cancelled_returncode

	async def _io_loop(self, input_file):
		background = self.background
		stdout_fd = self.stdout_fd
		log_file = self._log_file
		fd = input_file.fileno()

		while True:
			buf = self._read_buf(fd)

			if buf is None:
				# not a POLLIN event, EAGAIN, etc...
				future = self.scheduler.create_future()
				self.scheduler.add_reader(fd, future.set_result, None)
				try:
					await future
				finally:
					# The loop and input file may have been closed.
					if not self.scheduler.is_closed():
						future.done() or future.cancel()
						# Do not call remove_reader in cases where fd has
						# been closed and then re-allocated to a concurrent
						# coroutine as in bug 716636.
						if not input_file.closed:
							self.scheduler.remove_reader(fd)
				continue

			if not buf:
				# EOF
				return

			if not background and stdout_fd is not None:
				failures = 0
				stdout_buf = buf
				while stdout_buf:
					try:
						stdout_buf = \
							stdout_buf[os.write(stdout_fd, stdout_buf):]
					except OSError as e:
						if e.errno != errno.EAGAIN:
							raise
						del e
						failures += 1
						if failures > 50:
							# Avoid a potentially infinite loop. In
							# most cases, the failure count is zero
							# and it's unlikely to exceed 1.
							raise

						# This means that a subprocess has put an inherited
						# stdio file descriptor (typically stdin) into
						# O_NONBLOCK mode. This is not acceptable (see bug
						# #264435), so revert it. We need to use a loop
						# here since there's a race condition due to
						# parallel processes being able to change the
						# flags on the inherited file descriptor.
						# TODO: When possible, avoid having child processes
						# inherit stdio file descriptors from portage
						# (maybe it can't be avoided with
						# PROPERTIES=interactive).
						fcntl.fcntl(stdout_fd, fcntl.F_SETFL,
							fcntl.fcntl(stdout_fd,
							fcntl.F_GETFL) ^ os.O_NONBLOCK)

			if log_file is not None:
				if self._log_file_nb:
					# Use the _writer function which uses os.write, since the
					# log_file.write method looses data when an EAGAIN occurs.
					await _writer(log_file, buf)
				else:
					# For gzip.GzipFile instances, the above _writer function
					# will not work because data written directly to the file
					# descriptor bypasses compression.
					log_file.write(buf)
					log_file.flush()

	def _io_loop_done(self, future):
		try:
			future.result()
		except asyncio.CancelledError:
			self.cancel()
			self._was_cancelled()
		self.returncode = self.returncode or os.EX_OK
		self._async_wait()

	def _unregister(self):
		if self.input_fd is not None:
			if isinstance(self.input_fd, int):
				os.close(self.input_fd)
			elif not self.input_fd.closed:
				self.scheduler.remove_reader(self.input_fd.fileno())
				self.input_fd.close()
			self.input_fd = None

		if self._io_loop_task is not None:
			if not self.scheduler.is_closed():
				self._io_loop_task.done() or self._io_loop_task.cancel()
			self._io_loop_task = None

		if self.stdout_fd is not None:
			os.close(self.stdout_fd)
			self.stdout_fd = None

		if self._log_file is not None:
			if not self._log_file.closed:
				self.scheduler.remove_writer(self._log_file.fileno())
				self._log_file.close()
			self._log_file = None

		if self._log_file_real is not None:
			# Avoid "ResourceWarning: unclosed file" since python 3.2.
			self._log_file_real.close()
			self._log_file_real = None

		self._registered = False



# Copyright 2014-2018 Gentoo Foundation
# Distributed under the terms of the GNU General Public License v2

from portage.tests import TestCase
from portage.tests.resolver.ResolverPlayground import (ResolverPlayground,
	ResolverPlaygroundTestCase)

class SlotOperatorRebuildTestCase(TestCase):

	def testSlotOperatorRebuild(self):

		ebuilds = {

			"app-misc/A-1" : {
				"EAPI": "5",
				"SLOT": "0/1"
			},

			"app-misc/A-2" : {
				"EAPI": "5",
				"SLOT": "0/2"
			},

			"app-misc/B-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/A:="
			},

			"app-misc/C-0" : {
				"EAPI": "5",
				"RDEPEND": "|| ( app-misc/X app-misc/A:= )"
			},

			"app-misc/D-1" : {
				"EAPI": "6",
				"RDEPEND": "app-misc/E",
			},

			"app-misc/E-1" : {
				"EAPI": "6",
				"RDEPEND": "app-misc/F:=",
			},

			"app-misc/F-1" : {
				"EAPI": "6",
				"SLOT": "0/1"
			},

			"app-misc/F-2" : {
				"EAPI": "6",
				"SLOT": "0/2"
			},
		}

		binpkgs = {
			"app-misc/E-1" : {
				"EAPI": "6",
				"RDEPEND": "app-misc/F:0/1=",
			},
		}

		installed = {

			"app-misc/A-1" : {
				"EAPI": "5",
				"SLOT": "0/1"
			},

			"app-misc/B-0" : {
				"EAPI": "5",
				"RDEPEND": "app-misc/A:0/1="
			},

			"app-misc/C-0" : {
				"EAPI": "5",
				"RDEPEND": "|| ( app-misc/X app-misc/A:0/1= )"
			},

			"app-misc/F-2" : {
				"EAPI": "6",
				"SLOT": "0/2"
			},
		}

		world = ["app-misc/B", "app-misc/C"]

		test_cases = (

			# Test bug #522652, where the unsatisfiable app-misc/X
			# atom is selected, and the dependency is placed into
			# _initially_unsatisfied_deps where it is ignored, causing
			# the app-misc/C-0 rebuild to be missed.
			ResolverPlaygroundTestCase(
				["app-misc/A"],
				options = {"--dynamic-deps": "n"},
				success = True,
				ambiguous_merge_order = True,
				mergelist = ['app-misc/A-2', ('app-misc/B-0', 'app-misc/C-0')]
			),

			# Test bug #652938, where a binary package built against an
			# older subslot triggered downgrade of an installed package.
			# In this case we want to reject the app-misc/E-1 binary
			# package, and rebuild it against the installed instance of
			# app-misc/F.
			ResolverPlaygroundTestCase(
				["app-misc/D"],
				options = {'--usepkg': True},
				success = True,
				mergelist = ['app-misc/E-1', 'app-misc/D-1']
			),
		)

		playground = ResolverPlayground(ebuilds=ebuilds, binpkgs=binpkgs,
			installed=installed, world=world, debug=False)
		try:
			for test_case in test_cases:
				playground.run_TestCase(test_case)
				self.assertEqual(test_case.test_success, True, test_case.fail_msg)
		finally:
			playground.cleanup()



# validators.py Portage File Loader Code
# Copyright 2007-2020 Gentoo Authors

from portage.dep import isvalidatom

ValidAtomValidator = isvalidatom

def PackagesFileValidator(atom):
	""" This function mutates atoms that begin with - or *
	    It then checks to see if that atom is valid, and if
	    so returns True, else it returns False.

	    Args:
		atom: a string representing an atom such as sys-apps/portage-2.1
	"""
	if atom.startswith("*") or atom.startswith("-"):
		atom = atom[1:]
	if not isvalidatom(atom):
		return False
	return True



# Copyright 2013-2021 Gentoo Authors
# Distributed under the terms of the GNU General Public License v2

import collections
import errno
import logging
import random
import subprocess

import portage
from portage import _encodings, _unicode_encode
from portage import os
from portage.util import ensure_dirs
from portage.util._async.FileCopier import FileCopier
from portage.util._async.FileDigester import FileDigester
from portage.util._async.PipeLogger import PipeLogger
from portage.util._async.PopenProcess import PopenProcess
from _emerge.CompositeTask import CompositeTask

default_hash_name = portage.const.MANIFEST2_HASH_DEFAULT

# Use --no-check-certificate since Manifest digests should provide
# enough security, and certificates can be self-signed or whatnot.
default_fetchcommand = "wget -c -v -t 1 --passive-ftp --no-check-certificate --timeout=60 -O \"${DISTDIR}/${FILE}\" \"${URI}\""

class FetchTask(CompositeTask):

	__slots__ = ('distfile', 'digests', 'config', 'cpv',
		'restrict', 'uri_tuple', '_current_mirror',
		'_current_stat', '_fetch_tmp_dir_info', '_fetch_tmp_file',
		'_fs_mirror_stack', '_mirror_stack',
		'_previously_added',
		'_primaryuri_stack', '_log_path', '_tried_uris')

	def _start(self):

		if self.config.options.fetch_log_dir is not None and \
			not self.config.options.dry_run:
			self._log_path = os.path.join(
				self.config.options.fetch_log_dir,
				self.distfile + '.log')

		self._previously_added = True
		if self.config.distfiles_db is not None and \
			self.distfile not in self.config.distfiles_db:
			self._previously_added = False
			# Convert _pkg_str to str in order to prevent pickle problems.
			self.config.distfiles_db[self.distfile] = str(self.cpv)

		if self.config.content_db is not None:
			self.config.content_db.add(self.distfile)

		if not self._have_needed_digests():
			msg = "incomplete digests: %s" % " ".join(self.digests)
			self.scheduler.output(msg, background=self.background,
				log_path=self._log_path)
			self.config.log_failure("%s\t%s\t%s" %
				(self.cpv, self.distfile, msg))
			self.config.file_failures[self.distfile] = self.cpv
			self.returncode = os.EX_OK
			self._async_wait()
			return

		st = None
		for layout in self.config.layouts:
			distfile_path = os.path.join(
				self.config.options.distfiles,
				layout.get_path(self.distfile))
			try:
				st = os.stat(distfile_path)
			except OSError as e:
				if e.errno not in (errno.ENOENT, errno.ESTALE):
					msg = "%s stat failed in %s: %s" % \
						(self.distfile, "distfiles", e)
					self.scheduler.output(msg + '\n', background=True,
						log_path=self._log_path)
					logging.error(msg)
			else:
				break

		size_ok = st is not None and st.st_size == self.digests["size"]

		if not size_ok:
			if self.config.options.dry_run:
				if st is not None:
					logging.info(("dry-run: delete '%s' with "
						"wrong size from distfiles") % (self.distfile,))
			else:
				# Do the unlink in order to ensure that the path is clear,
				# even if stat raised ENOENT, since a broken symlink can
				# trigger ENOENT.
				unlink_success = True
				for layout in self.config.layouts:
					unlink_path = os.path.join(
						self.config.options.distfiles,
						layout.get_path(self.distfile))
					if self._unlink_file(unlink_path, "distfiles"):
						if st is not None:
							logging.debug(("delete '%s' with "
								"wrong size from distfiles") % (self.distfile,))
					else:
						self.config.log_failure("%s\t%s\t%s" %
							(self.cpv, self.distfile, "unlink failed in distfiles"))
						unlink_success = False
				if not unlink_success:
					self.returncode = os.EX_OK
					self._async_wait()
					return

		if size_ok:
			if self.config.options.verify_existing_digest:
				self._start_task(
					FileDigester(file_path=distfile_path,
						hash_names=(self._select_hash(),),
						background=self.background,
						logfile=self._log_path), self._distfiles_digester_exit)
				return

			self._success()
			self.returncode = os.EX_OK
			self._async_wait()
			return

		self._start_fetch()

	def _success(self):
		if not self._previously_added:
			size = self.digests["size"]
			self.config.added_byte_count += size
			self.config.added_file_count += 1
			self.config.log_success("%s\t%s\tadded %i bytes" %
				(self.cpv, self.distfile, size))

		if self._log_path is not None:
			if not self.config.options.dry_run:
				try:
					os.unlink(self._log_path)
				except OSError:
					pass

		if self.config.options.recycle_dir is not None:

			recycle_file = os.path.join(
				self.config.options.recycle_dir, self.distfile)

			if self.config.options.dry_run:
				if os.path.exists(recycle_file):
					logging.info("dry-run: delete '%s' from recycle" %
						(self.distfile,))
			else:
				try:
					os.unlink(recycle_file)
				except OSError:
					pass
				else:
					logging.debug("delete '%s' from recycle" %
						(self.distfile,))

	def _distfiles_digester_exit(self, digester):

		self._assert_current(digester)
		if self._was_cancelled():
			self.wait()
			return

		if self._default_exit(digester) != os.EX_OK:
			# IOError reading file in our main distfiles directory? This
			# is a bad situation which normally does not occur, so
			# skip this file and report it, in order to draw attention
			# from the administrator.
			msg = "%s distfiles digester failed unexpectedly" % \
				(self.distfile,)
			self.scheduler.output(msg + '\n', background=True,
				log_path=self._log_path)
			logging.error(msg)
			self.config.log_failure("%s\t%s\t%s" %
				(self.cpv, self.distfile, msg))
			self.config.file_failures[self.distfile] = self.cpv
			self.wait()
			return

		wrong_digest = self._find_bad_digest(digester.digests)
		if wrong_digest is None:
			self._success()
			self.returncode = os.EX_OK
			self.wait()
			return

		self._start_fetch()

	_mirror_info = collections.namedtuple('_mirror_info',
		'name location')

	def _start_fetch(self):

		self._previously_added = False
		self._fs_mirror_stack = []
		if self.config.options.distfiles_local is not None:
			self._fs_mirror_stack.append(self._mirror_info(
				'distfiles-local', self.config.options.distfiles_local))
		if self.config.options.recycle_dir is not None:
			self._fs_mirror_stack.append(self._mirror_info(
				'recycle', self.config.options.recycle_dir))

		self._primaryuri_stack = []
		self._mirror_stack = []
		for uri in reversed(self.uri_tuple):
			if uri.startswith('mirror://'):
				self._mirror_stack.append(
					self._mirror_iterator(uri, self.config.mirrors))
			else:
				self._primaryuri_stack.append(uri)

		self._tried_uris = set()
		self._try_next_mirror()

	@staticmethod
	def _mirror_iterator(uri, mirrors_dict):

		slash_index = uri.find("/", 9)
		if slash_index != -1:
			mirror_name = uri[9:slash_index].strip("/")
			mirrors = mirrors_dict.get(mirror_name)
			if not mirrors:
				return
			mirrors = list(mirrors)
			while mirrors:
				mirror = mirrors.pop(random.randint(0, len(mirrors) - 1))
				yield mirror.rstrip("/") + "/" + uri[slash_index+1:]

	def _try_next_mirror(self):
		if self._fs_mirror_stack:
			self._fetch_fs(self._fs_mirror_stack.pop())
			return
		uri = self._next_uri()
		if uri is not None:
			self._tried_uris.add(uri)
			self._fetch_uri(uri)
			return

		if self._tried_uris:
			msg = "all uris failed"
		else:
			msg = "no fetchable uris"

		self.config.log_failure("%s\t%s\t%s" %
			(self.cpv, self.distfile, msg))
		self.config.file_failures[self.distfile] = self.cpv
		self.returncode = os.EX_OK
		self.wait()

	def _next_uri(self):
		remaining_tries = self.config.options.tries - len(self._tried_uris)
		if remaining_tries > 0:

			if remaining_tries <= self.config.options.tries // 2:
				while self._primaryuri_stack:
					uri = self._primaryuri_stack.pop()
					if uri not in self._tried_uris:
						return uri

			while self._mirror_stack:
				uri = next(self._mirror_stack[-1], None)
				if uri is None:
					self._mirror_stack.pop()
				else:
					if uri not in self._tried_uris:
						return uri

			while self._primaryuri_stack:
				uri = self._primaryuri_stack.pop()
				if uri not in self._tried_uris:
					return uri

		return None

	def _fetch_fs(self, mirror_info):
		file_path = os.path.join(mirror_info.location, self.distfile)

		st = None
		size_ok = False
		try:
			st = os.stat(file_path)
		except OSError as e:
			if e.errno not in (errno.ENOENT, errno.ESTALE):
				msg = "%s stat failed in %s: %s" % \
					(self.distfile, mirror_info.name, e)
				self.scheduler.output(msg + '\n', background=True,
					log_path=self._log_path)
				logging.error(msg)
		else:
			size_ok = st.st_size == self.digests["size"]
			self._current_stat = st

		if size_ok:
			self._current_mirror = mirror_info
			self._start_task(
				FileDigester(file_path=file_path,
					hash_names=(self._select_hash(),),
					background=self.background,
					logfile=self._log_path),
				self._fs_mirror_digester_exit)
		else:
			self._try_next_mirror()

	def _fs_mirror_digester_exit(self, digester):

		self._assert_current(digester)
		if self._was_cancelled():
			self.wait()
			return

		current_mirror = self._current_mirror
		if digester.returncode != os.EX_OK:
			msg = "%s %s digester failed unexpectedly" % \
			(self.distfile, current_mirror.name)
			self.scheduler.output(msg + '\n', background=True,
				log_path=self._log_path)
			logging.error(msg)
		else:
			bad_digest = self._find_bad_digest(digester.digests)
			if bad_digest is not None:
				msg = "%s %s has bad %s digest: expected %s, got %s" % \
					(self.distfile, current_mirror.name, bad_digest,
					self.digests[bad_digest], digester.digests[bad_digest])
				self.scheduler.output(msg + '\n', background=True,
					log_path=self._log_path)
				logging.error(msg)
			elif self.config.options.dry_run:
				# Report success without actually touching any files
				if self._same_device(current_mirror.location,
					self.config.options.distfiles):
					logging.info(("dry-run: hardlink '%s' from %s "
						"to distfiles") % (self.distfile, current_mirror.name))
				else:
					logging.info("dry-run: copy '%s' from %s to distfiles" %
						(self.distfile, current_mirror.name))
				self._success()
				self.returncode = os.EX_OK
				self.wait()
				return
			else:
				src = os.path.join(current_mirror.location, self.distfile)
				dest = os.path.join(self.config.options.distfiles,
						self.config.layouts[0].get_path(self.distfile))
				if self._hardlink_atomic(src, dest,
					"%s to %s" % (current_mirror.name, "distfiles")):
					logging.debug("hardlink '%s' from %s to distfiles" %
						(self.distfile, current_mirror.name))
					self._success()
					self.returncode = os.EX_OK
					self.wait()
					return

				self._start_task(
					FileCopier(src_path=src, dest_path=dest,
						background=(self.background and
							self._log_path is not None),
						logfile=self._log_path),
					self._fs_mirror_copier_exit)
				return

		self._try_next_mirror()

	def _fs_mirror_copier_exit(self, copier):

		self._assert_current(copier)
		if self._was_cancelled():
			self.wait()
			return

		current_mirror = self._current_mirror
		if copier.returncode != os.EX_OK:
			msg = "%s %s copy failed unexpectedly: %s" % \
				(self.distfile, current_mirror.name, copier.future.exception())
			self.scheduler.output(msg + '\n', background=True,
				log_path=self._log_path)
			logging.error(msg)
		else:

			logging.debug("copy '%s' from %s to distfiles" %
				(self.distfile, current_mirror.name))

			# Apply the timestamp from the source file, but
			# just rely on umask for permissions.
			try:
				os.utime(copier.dest_path,
					ns=(self._current_stat.st_mtime_ns,
					self._current_stat.st_mtime_ns))
			except OSError as e:
				msg = "%s %s utime failed unexpectedly: %s" % \
					(self.distfile, current_mirror.name, e)
				self.scheduler.output(msg + '\n', background=True,
					log_path=self._log_path)
				logging.error(msg)

			self._success()
			self.returncode = os.EX_OK
			self.wait()
			return

		self._try_next_mirror()

	def _fetch_uri(self, uri):

		if self.config.options.dry_run:
			# Simply report success.
			logging.info("dry-run: fetch '%s' from '%s'" %
				(self.distfile, uri))
			self._success()
			self.returncode = os.EX_OK
			self._async_wait()
			return

		if self.config.options.temp_dir:
			self._fetch_tmp_dir_info = 'temp-dir'
			distdir = self.config.options.temp_dir
		else:
			self._fetch_tmp_dir_info = 'distfiles'
			distdir = self.config.options.distfiles

		tmp_basename = self.distfile + '._emirrordist_fetch_.%s' % portage.getpid()

		variables = {
			"DISTDIR": distdir,
			"URI":     uri,
			"FILE":    tmp_basename
		}

		self._fetch_tmp_file = os.path.join(distdir, tmp_basename)

		try:
			os.unlink(self._fetch_tmp_file)
		except OSError:
			pass

		args = portage.util.shlex_split(default_fetchcommand)
		args = [portage.util.varexpand(x, mydict=variables)
			for x in args]

		args = [_unicode_encode(x,
			encoding=_encodings['fs'], errors='strict') for x in args]

		null_fd = os.open(os.devnull, os.O_RDONLY)
		fetcher = PopenProcess(background=self.background,
			proc=subprocess.Popen(args, stdin=null_fd,
			stdout=subprocess.PIPE, stderr=subprocess.STDOUT),
			scheduler=self.scheduler)
		os.close(null_fd)

		fetcher.pipe_reader = PipeLogger(background=self.background,
			input_fd=fetcher.proc.stdout, log_file_path=self._log_path,
			scheduler=self.scheduler)

		self._start_task(fetcher, self._fetcher_exit)

	def _fetcher_exit(self, fetcher):

		self._assert_current(fetcher)
		if self._was_cancelled():
			self.wait()
			return

		if os.path.exists(self._fetch_tmp_file):
			self._start_task(
				FileDigester(file_path=self._fetch_tmp_file,
					hash_names=(self._select_hash(),),
					background=self.background,
					logfile=self._log_path),
					self._fetch_digester_exit)
		else:
			self._try_next_mirror()

	def _fetch_digester_exit(self, digester):

		self._assert_current(digester)
		if self._was_cancelled():
			self.wait()
			return

		if digester.returncode != os.EX_OK:
			msg = "%s %s digester failed unexpectedly" % \
			(self.distfile, self._fetch_tmp_dir_info)
			self.scheduler.output(msg + '\n', background=True,
				log_path=self._log_path)
			logging.error(msg)
		else:
			bad_digest = self._find_bad_digest(digester.digests)
			if bad_digest is not None:
				msg = "%s has bad %s digest: expected %s, got %s" % \
					(self.distfile, bad_digest,
					self.digests[bad_digest], digester.digests[bad_digest])
				self.scheduler.output(msg + '\n', background=True,
					log_path=self._log_path)
				try:
					os.unlink(self._fetch_tmp_file)
				except OSError:
					pass
			else:
				dest = os.path.join(self.config.options.distfiles,
						self.config.layouts[0].get_path(self.distfile))
				ensure_dirs(os.path.dirname(dest))
				try:
					os.rename(self._fetch_tmp_file, dest)
				except OSError:
					self._start_task(
						FileCopier(src_path=self._fetch_tmp_file,
							dest_path=dest,
							background=(self.background and
								self._log_path is not None),
							logfile=self._log_path),
						self._fetch_copier_exit)
					return
				else:
					self._make_layout_links()
					return

		self._try_next_mirror()

	def _fetch_copier_exit(self, copier):

		self._assert_current(copier)

		try:
			os.unlink(self._fetch_tmp_file)
		except OSError:
			pass

		if self._was_cancelled():
			self.wait()
			return

		if copier.returncode == os.EX_OK:
			self._make_layout_links()
		else:
			# out of space?
			msg = "%s %s copy failed unexpectedly: %s" % \
				(self.distfile, self._fetch_tmp_dir_info, copier.future.exception())
			self.scheduler.output(msg + '\n', background=True,
				log_path=self._log_path)
			logging.error(msg)
			self.config.log_failure("%s\t%s\t%s" %
				(self.cpv, self.distfile, msg))
			self.config.file_failures[self.distfile] = self.cpv
			self.returncode = 1
			self.wait()

	def _make_layout_links(self):
		dist_path = None
		success = True
		for layout in self.config.layouts[1:]:
			if dist_path is None:
				dist_path = os.path.join(self.config.options.distfiles,
						self.config.layouts[0].get_path(self.distfile))
			link_path = os.path.join(self.config.options.distfiles,
					layout.get_path(self.distfile))
			ensure_dirs(os.path.dirname(link_path))
			src_path = dist_path
			if self.config.options.symlinks:
				src_path = os.path.relpath(dist_path,
						os.path.dirname(link_path))

			if not self._hardlink_atomic(src_path, link_path,
					"%s -> %s" % (link_path, src_path),
					self.config.options.symlinks):
				success = False
				break

		if success:
			self._success()
			self.returncode = os.EX_OK
		else:
			msg = "failed to create distfiles layout {}".format(
				"symlink" if self.config.options.symlinks else "hardlink")
			self.config.log_failure("%s\t%s\t%s" %
				(self.cpv, self.distfile, msg))
			self.config.file_failures[self.distfile] = self.cpv
			self.returncode = 1

		self.wait()

	def _unlink_file(self, file_path, dir_info):
		try:
			os.unlink(file_path)
		except OSError as e:
			if e.errno not in (errno.ENOENT, errno.ESTALE):
				msg = "unlink '%s' failed in %s: %s" % \
					(self.distfile, dir_info, e)
				self.scheduler.output(msg + '\n', background=True,
					log_path=self._log_path)
				logging.error(msg)
				return False
		return True

	def _have_needed_digests(self):
		return "size" in self.digests and \
			self._select_hash() is not None

	def _select_hash(self):
		if default_hash_name in self.digests:
			return default_hash_name
		for hash_name in self.digests:
			if hash_name != "size" and \
				hash_name in portage.checksum.get_valid_checksum_keys():
				return hash_name

		return None

	def _find_bad_digest(self, digests):
		for hash_name, hash_value in digests.items():
			if self.digests[hash_name] != hash_value:
				return hash_name
		return None

	@staticmethod
	def _same_device(path1, path2):
		try:
			st1 = os.stat(path1)
			st2 = os.stat(path2)
		except OSError:
			return False
		else:
			return st1.st_dev == st2.st_dev

	def _hardlink_atomic(self, src, dest, dir_info, symlink=False):

		head, tail = os.path.split(dest)
		hardlink_tmp = os.path.join(head, ".%s._mirrordist_hardlink_.%s" % \
			(tail, portage.getpid()))

		try:
			try:
				if symlink:
					os.symlink(src, hardlink_tmp)
				else:
					os.link(src, hardlink_tmp)
			except OSError as e:
				if e.errno != errno.EXDEV:
					msg = "hardlink %s from %s failed: %s" % \
						(self.distfile, dir_info, e)
					self.scheduler.output(msg + '\n', background=True,
						log_path=self._log_path)
					logging.error(msg)
				return False

			try:
				os.rename(hardlink_tmp, dest)
			except OSError as e:
				msg = "hardlink rename '%s' from %s failed: %s" % \
					(self.distfile, dir_info, e)
				self.scheduler.output(msg + '\n', background=True,
					log_path=self._log_path)
				logging.error(msg)
				return False
		finally:
			try:
				os.unlink(hardlink_tmp)
			except OSError:
				pass

		return True
